# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './cnmfe_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.setWindowModality(QtCore.Qt.NonModal)
        DockWidget.resize(997, 828)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout(self.dockWidgetContents)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_2.addWidget(self.label_5)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxGSig = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGSig.setMinimum(2)
        self.spinBoxGSig.setMaximum(998)
        self.spinBoxGSig.setSingleStep(1)
        self.spinBoxGSig.setProperty("value", 10)
        self.spinBoxGSig.setObjectName("spinBoxGSig")
        self.horizontalLayout.addWidget(self.spinBoxGSig)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_3.addWidget(self.label_9)
        self.lineEdCorrPNRName = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEdCorrPNRName.setObjectName("lineEdCorrPNRName")
        self.horizontalLayout_3.addWidget(self.lineEdCorrPNRName)
        self.btnAddToBatchCorrPNR = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchCorrPNR.setObjectName("btnAddToBatchCorrPNR")
        self.horizontalLayout_3.addWidget(self.btnAddToBatchCorrPNR)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.line_2 = QtWidgets.QFrame(self.dockWidgetContents)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_2.addWidget(self.line_2)
        self.label_20 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.verticalLayout_2.addWidget(self.label_20)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.lineEditAin = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEditAin.setObjectName("lineEditAin")
        self.horizontalLayout_2.addWidget(self.lineEditAin)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_19 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_12.addWidget(self.label_19)
        self.spinBox_p = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_p.setProperty("value", 1)
        self.spinBox_p.setObjectName("spinBox_p")
        self.horizontalLayout_12.addWidget(self.spinBox_p)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_12)
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_2.addWidget(self.label_7)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_8 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_6.addWidget(self.label_8)
        self.doubleSpinBoxMinCorr = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxMinCorr.setDecimals(3)
        self.doubleSpinBoxMinCorr.setMaximum(1.0)
        self.doubleSpinBoxMinCorr.setSingleStep(0.005)
        self.doubleSpinBoxMinCorr.setProperty("value", 0.89)
        self.doubleSpinBoxMinCorr.setObjectName("doubleSpinBoxMinCorr")
        self.horizontalLayout_6.addWidget(self.doubleSpinBoxMinCorr)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.label_11 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_2.addWidget(self.label_11)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_10 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_7.addWidget(self.label_10)
        self.spinBoxMinPNR = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxMinPNR.setMaximum(999)
        self.spinBoxMinPNR.setProperty("value", 4)
        self.spinBoxMinPNR.setObjectName("spinBoxMinPNR")
        self.horizontalLayout_7.addWidget(self.spinBoxMinPNR)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.label_13 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_13.setObjectName("label_13")
        self.verticalLayout_2.addWidget(self.label_13)
        self.label_24 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_24.setObjectName("label_24")
        self.verticalLayout_2.addWidget(self.label_24)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_21 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_13.addWidget(self.label_21)
        self.spinBoxRf = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxRf.setMaximum(999)
        self.spinBoxRf.setProperty("value", 50)
        self.spinBoxRf.setObjectName("spinBoxRf")
        self.horizontalLayout_13.addWidget(self.spinBoxRf)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_13)
        self.label_25 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_25.setObjectName("label_25")
        self.verticalLayout_2.addWidget(self.label_25)
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_26 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_18.addWidget(self.label_26)
        self.spinBoxOverlap = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxOverlap.setMaximum(999)
        self.spinBoxOverlap.setProperty("value", 30)
        self.spinBoxOverlap.setObjectName("spinBoxOverlap")
        self.horizontalLayout_18.addWidget(self.spinBoxOverlap)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_18)
        self.label_28 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_28.setObjectName("label_28")
        self.verticalLayout_2.addWidget(self.label_28)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_27 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_27.setObjectName("label_27")
        self.horizontalLayout_19.addWidget(self.label_27)
        self.spinBoxGnb = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGnb.setMaximum(99)
        self.spinBoxGnb.setProperty("value", 0)
        self.spinBoxGnb.setObjectName("spinBoxGnb")
        self.horizontalLayout_19.addWidget(self.spinBoxGnb)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_19)
        self.label_30 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_30.setObjectName("label_30")
        self.verticalLayout_2.addWidget(self.label_30)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.label_29 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_29.setObjectName("label_29")
        self.horizontalLayout_20.addWidget(self.label_29)
        self.spinBoxNb_patch = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxNb_patch.setMinimum(0)
        self.spinBoxNb_patch.setProperty("value", 0)
        self.spinBoxNb_patch.setObjectName("spinBoxNb_patch")
        self.horizontalLayout_20.addWidget(self.spinBoxNb_patch)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_20.addItem(spacerItem7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_20)
        self.label_32 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_32.setObjectName("label_32")
        self.verticalLayout_2.addWidget(self.label_32)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_31 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_31.setObjectName("label_31")
        self.horizontalLayout_21.addWidget(self.label_31)
        self.spinBoxK = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxK.setMinimum(0)
        self.spinBoxK.setProperty("value", 10)
        self.spinBoxK.setObjectName("spinBoxK")
        self.horizontalLayout_21.addWidget(self.spinBoxK)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_21)
        self.horizontalLayout_15.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_23 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_23.setObjectName("horizontalLayout_23")
        self.label_35 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_35.setObjectName("label_35")
        self.horizontalLayout_23.addWidget(self.label_35)
        self.spinBox_ssub = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_ssub.setMinimum(1)
        self.spinBox_ssub.setProperty("value", 1)
        self.spinBox_ssub.setObjectName("spinBox_ssub")
        self.horizontalLayout_23.addWidget(self.spinBox_ssub)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_23.addWidget(self.line)
        self.label_36 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_36.setObjectName("label_36")
        self.horizontalLayout_23.addWidget(self.label_36)
        self.spinBox_tsub = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_tsub.setMinimum(1)
        self.spinBox_tsub.setSingleStep(1)
        self.spinBox_tsub.setProperty("value", 1)
        self.spinBox_tsub.setObjectName("spinBox_tsub")
        self.horizontalLayout_23.addWidget(self.spinBox_tsub)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_23.addItem(spacerItem9)
        self.verticalLayout.addLayout(self.horizontalLayout_23)
        self.checkBox_low_rank_background = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBox_low_rank_background.setObjectName("checkBox_low_rank_background")
        self.verticalLayout.addWidget(self.checkBox_low_rank_background)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_4.addWidget(self.label_6)
        self.doubleSpinBox_ring_size_factor = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBox_ring_size_factor.setSingleStep(0.1)
        self.doubleSpinBox_ring_size_factor.setProperty("value", 1.5)
        self.doubleSpinBox_ring_size_factor.setObjectName("doubleSpinBox_ring_size_factor")
        self.horizontalLayout_4.addWidget(self.doubleSpinBox_ring_size_factor)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem10)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.comboBoxDeconv = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxDeconv.setObjectName("comboBoxDeconv")
        self.comboBoxDeconv.addItem("")
        self.comboBoxDeconv.addItem("")
        self.comboBoxDeconv.addItem("")
        self.horizontalLayout_5.addWidget(self.comboBoxDeconv)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem11)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_22 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_22.setObjectName("label_22")
        self.horizontalLayout_14.addWidget(self.label_22)
        self.doubleSpinBoxMergeThresh = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxMergeThresh.setMaximum(1.0)
        self.doubleSpinBoxMergeThresh.setSingleStep(0.05)
        self.doubleSpinBoxMergeThresh.setProperty("value", 0.7)
        self.doubleSpinBoxMergeThresh.setObjectName("doubleSpinBoxMergeThresh")
        self.horizontalLayout_14.addWidget(self.doubleSpinBoxMergeThresh)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem12)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.groupBox_cnmf_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_cnmf_kwargs.setStatusTip("")
        self.groupBox_cnmf_kwargs.setCheckable(True)
        self.groupBox_cnmf_kwargs.setChecked(False)
        self.groupBox_cnmf_kwargs.setObjectName("groupBox_cnmf_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_cnmf_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_cnmf_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_cnmf_kwargs)
        self.plainTextEdit_cnmf_kwargs.setObjectName("plainTextEdit_cnmf_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_cnmf_kwargs)
        self.verticalLayout.addWidget(self.groupBox_cnmf_kwargs)
        self.label_23 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setObjectName("label_23")
        self.verticalLayout.addWidget(self.label_23)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_12 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_8.addWidget(self.label_12)
        self.spinBoxMinSNR = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxMinSNR.setProperty("value", 1)
        self.spinBoxMinSNR.setObjectName("spinBoxMinSNR")
        self.horizontalLayout_8.addWidget(self.spinBoxMinSNR)
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem13)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.label_14 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_14.setObjectName("label_14")
        self.verticalLayout.addWidget(self.label_14)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_15 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_9.addWidget(self.label_15)
        self.doubleSpinBoxRValuesMin = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxRValuesMin.setMaximum(1.0)
        self.doubleSpinBoxRValuesMin.setSingleStep(0.025)
        self.doubleSpinBoxRValuesMin.setProperty("value", 0.7)
        self.doubleSpinBoxRValuesMin.setObjectName("doubleSpinBoxRValuesMin")
        self.horizontalLayout_9.addWidget(self.doubleSpinBoxRValuesMin)
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem14)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        self.label_17 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_17.setObjectName("label_17")
        self.verticalLayout.addWidget(self.label_17)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_16 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_10.addWidget(self.label_16)
        self.doubleSpinBoxDecayTime = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxDecayTime.setMinimum(0.1)
        self.doubleSpinBoxDecayTime.setSingleStep(0.5)
        self.doubleSpinBoxDecayTime.setProperty("value", 4.0)
        self.doubleSpinBoxDecayTime.setObjectName("doubleSpinBoxDecayTime")
        self.horizontalLayout_10.addWidget(self.doubleSpinBoxDecayTime)
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem15)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.groupBox_eval_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_eval_kwargs.setCheckable(True)
        self.groupBox_eval_kwargs.setChecked(False)
        self.groupBox_eval_kwargs.setObjectName("groupBox_eval_kwargs")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_eval_kwargs)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.plainTextEdit_eval_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_eval_kwargs)
        self.plainTextEdit_eval_kwargs.setObjectName("plainTextEdit_eval_kwargs")
        self.verticalLayout_5.addWidget(self.plainTextEdit_eval_kwargs)
        self.verticalLayout.addWidget(self.groupBox_eval_kwargs)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_18 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_11.addWidget(self.label_18)
        self.lineEdName = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEdName.setObjectName("lineEdName")
        self.horizontalLayout_11.addWidget(self.lineEdName)
        self.btnAddToBatchCNMFE = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchCNMFE.setObjectName("btnAddToBatchCNMFE")
        self.horizontalLayout_11.addWidget(self.btnAddToBatchCNMFE)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_15.addLayout(self.verticalLayout)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.lineEdName.returnPressed.connect(self.btnAddToBatchCNMFE.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.spinBoxGSig, self.lineEdCorrPNRName)
        DockWidget.setTabOrder(self.lineEdCorrPNRName, self.btnAddToBatchCorrPNR)
        DockWidget.setTabOrder(self.btnAddToBatchCorrPNR, self.lineEditAin)
        DockWidget.setTabOrder(self.lineEditAin, self.spinBox_p)
        DockWidget.setTabOrder(self.spinBox_p, self.doubleSpinBoxMinCorr)
        DockWidget.setTabOrder(self.doubleSpinBoxMinCorr, self.spinBoxMinPNR)
        DockWidget.setTabOrder(self.spinBoxMinPNR, self.spinBoxRf)
        DockWidget.setTabOrder(self.spinBoxRf, self.spinBoxOverlap)
        DockWidget.setTabOrder(self.spinBoxOverlap, self.spinBoxGnb)
        DockWidget.setTabOrder(self.spinBoxGnb, self.spinBoxNb_patch)
        DockWidget.setTabOrder(self.spinBoxNb_patch, self.spinBoxK)
        DockWidget.setTabOrder(self.spinBoxK, self.spinBox_ssub)
        DockWidget.setTabOrder(self.spinBox_ssub, self.spinBox_tsub)
        DockWidget.setTabOrder(self.spinBox_tsub, self.checkBox_low_rank_background)
        DockWidget.setTabOrder(self.checkBox_low_rank_background, self.doubleSpinBox_ring_size_factor)
        DockWidget.setTabOrder(self.doubleSpinBox_ring_size_factor, self.comboBoxDeconv)
        DockWidget.setTabOrder(self.comboBoxDeconv, self.doubleSpinBoxMergeThresh)
        DockWidget.setTabOrder(self.doubleSpinBoxMergeThresh, self.groupBox_cnmf_kwargs)
        DockWidget.setTabOrder(self.groupBox_cnmf_kwargs, self.plainTextEdit_cnmf_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_cnmf_kwargs, self.spinBoxMinSNR)
        DockWidget.setTabOrder(self.spinBoxMinSNR, self.doubleSpinBoxRValuesMin)
        DockWidget.setTabOrder(self.doubleSpinBoxRValuesMin, self.doubleSpinBoxDecayTime)
        DockWidget.setTabOrder(self.doubleSpinBoxDecayTime, self.groupBox_eval_kwargs)
        DockWidget.setTabOrder(self.groupBox_eval_kwargs, self.plainTextEdit_eval_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_eval_kwargs, self.lineEdName)
        DockWidget.setTabOrder(self.lineEdName, self.btnAddToBatchCNMFE)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&CNMF-E"))
        self.label_5.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_5.setText(_translate("DockWidget", "Inspect Correlation and PNR"))
        self.label_2.setText(_translate("DockWidget", "gaussian width of a 2D gaussian kernel, which approximates a neuron\n"
"3 times less than the average diamters of a neuron (pixels)"))
        self.label.setText(_translate("DockWidget", "gSig:"))
        self.label_9.setText(_translate("DockWidget", "Stop here:"))
        self.lineEdCorrPNRName.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchCorrPNR.setText(_translate("DockWidget", "Add to batch"))
        self.label_20.setText(_translate("DockWidget", "CNMF-E"))
        self.label_3.setText(_translate("DockWidget", "Ain:"))
        self.label_19.setText(_translate("DockWidget", "p:"))
        self.label_7.setText(_translate("DockWidget", "Minimum correlation of peak"))
        self.label_8.setText(_translate("DockWidget", "min_corr:"))
        self.label_11.setText(_translate("DockWidget", "Minimum peak to noise ratio"))
        self.label_10.setText(_translate("DockWidget", "min_pnr:"))
        self.label_13.setText(_translate("DockWidget", "Adaptive way to set threshold on the transient size"))
        self.label_24.setText(_translate("DockWidget", "Half size of patch"))
        self.label_21.setText(_translate("DockWidget", "rf:"))
        self.label_25.setText(_translate("DockWidget", "Overlap of patches (at least 4 times the size of a neuron/cell)"))
        self.label_26.setText(_translate("DockWidget", "overlap:"))
        self.label_28.setText(_translate("DockWidget", "Global number of background components"))
        self.label_27.setText(_translate("DockWidget", "gnb:"))
        self.label_30.setText(_translate("DockWidget", "Background components per patch"))
        self.label_29.setText(_translate("DockWidget", "nb_patch:"))
        self.label_32.setText(_translate("DockWidget", "Number of neurons/cell per patch"))
        self.label_31.setText(_translate("DockWidget", "k:"))
        self.label_35.setText(_translate("DockWidget", "ssub:"))
        self.label_36.setText(_translate("DockWidget", "tsub:"))
        self.checkBox_low_rank_background.setText(_translate("DockWidget", "low_rank_background"))
        self.label_6.setText(_translate("DockWidget", "ring_size_factor:"))
        self.label_4.setText(_translate("DockWidget", "deconvolution: "))
        self.comboBoxDeconv.setItemText(0, _translate("DockWidget", "oasis"))
        self.comboBoxDeconv.setItemText(1, _translate("DockWidget", "cvxpy"))
        self.comboBoxDeconv.setItemText(2, _translate("DockWidget", "SKIP"))
        self.label_22.setText(_translate("DockWidget", "merge_thresh:"))
        self.groupBox_cnmf_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_cnmf_kwargs.setTitle(_translate("DockWidget", "Use CNMF &kwargs"))
        self.plainTextEdit_cnmf_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_23.setText(_translate("DockWidget", "Evalulation params"))
        self.label_12.setText(_translate("DockWidget", "min_SNR:"))
        self.label_14.setText(_translate("DockWidget", "Threshold on space consistency \n"
"If lower more components will be accepted, potentially with worse quality"))
        self.label_15.setText(_translate("DockWidget", "r_values_min:"))
        self.label_17.setText(_translate("DockWidget", "Average decay time of calcium spikes (seconds)"))
        self.label_16.setText(_translate("DockWidget", "decay_time:"))
        self.groupBox_eval_kwargs.setToolTip(_translate("DockWidget", "You can enter additional parameters to use for component evaluation.\n"
"Use single quotes for strings, do not use double quotes"))
        self.groupBox_eval_kwargs.setTitle(_translate("DockWidget", "Use e&valuation params"))
        self.plainTextEdit_eval_kwargs.setToolTip(_translate("DockWidget", "You can enter additional parameters to use for component evaluation.\n"
"Use single quotes for strings, do not use double quotes"))
        self.label_18.setText(_translate("DockWidget", "Perform CNMF-E:"))
        self.lineEdName.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchCNMFE.setText(_translate("DockWidget", "Add to batch"))


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './cnmfe_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.setWindowModality(QtCore.Qt.NonModal)
        DockWidget.resize(997, 828)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout(self.dockWidgetContents)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_2.addWidget(self.label_5)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxGSig = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGSig.setMinimum(2)
        self.spinBoxGSig.setMaximum(998)
        self.spinBoxGSig.setSingleStep(1)
        self.spinBoxGSig.setProperty("value", 10)
        self.spinBoxGSig.setObjectName("spinBoxGSig")
        self.horizontalLayout.addWidget(self.spinBoxGSig)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_9 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_3.addWidget(self.label_9)
        self.lineEdCorrPNRName = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEdCorrPNRName.setObjectName("lineEdCorrPNRName")
        self.horizontalLayout_3.addWidget(self.lineEdCorrPNRName)
        self.btnAddToBatchCorrPNR = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchCorrPNR.setObjectName("btnAddToBatchCorrPNR")
        self.horizontalLayout_3.addWidget(self.btnAddToBatchCorrPNR)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.line_2 = QtWidgets.QFrame(self.dockWidgetContents)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_2.addWidget(self.line_2)
        self.label_20 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.verticalLayout_2.addWidget(self.label_20)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.lineEditAin = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEditAin.setObjectName("lineEditAin")
        self.horizontalLayout_2.addWidget(self.lineEditAin)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_19 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_12.addWidget(self.label_19)
        self.spinBox_p = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_p.setProperty("value", 1)
        self.spinBox_p.setObjectName("spinBox_p")
        self.horizontalLayout_12.addWidget(self.spinBox_p)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_12)
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_2.addWidget(self.label_7)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_8 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_6.addWidget(self.label_8)
        self.doubleSpinBoxMinCorr = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxMinCorr.setDecimals(3)
        self.doubleSpinBoxMinCorr.setMaximum(1.0)
        self.doubleSpinBoxMinCorr.setSingleStep(0.005)
        self.doubleSpinBoxMinCorr.setProperty("value", 0.89)
        self.doubleSpinBoxMinCorr.setObjectName("doubleSpinBoxMinCorr")
        self.horizontalLayout_6.addWidget(self.doubleSpinBoxMinCorr)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.label_11 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_2.addWidget(self.label_11)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_10 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_7.addWidget(self.label_10)
        self.spinBoxMinPNR = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxMinPNR.setMaximum(999)
        self.spinBoxMinPNR.setProperty("value", 4)
        self.spinBoxMinPNR.setObjectName("spinBoxMinPNR")
        self.horizontalLayout_7.addWidget(self.spinBoxMinPNR)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.label_13 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_13.setObjectName("label_13")
        self.verticalLayout_2.addWidget(self.label_13)
        self.label_24 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_24.setObjectName("label_24")
        self.verticalLayout_2.addWidget(self.label_24)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_21 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_13.addWidget(self.label_21)
        self.spinBoxRf = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxRf.setMaximum(999)
        self.spinBoxRf.setProperty("value", 50)
        self.spinBoxRf.setObjectName("spinBoxRf")
        self.horizontalLayout_13.addWidget(self.spinBoxRf)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_13)
        self.label_25 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_25.setObjectName("label_25")
        self.verticalLayout_2.addWidget(self.label_25)
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_26 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_18.addWidget(self.label_26)
        self.spinBoxOverlap = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxOverlap.setMaximum(999)
        self.spinBoxOverlap.setProperty("value", 30)
        self.spinBoxOverlap.setObjectName("spinBoxOverlap")
        self.horizontalLayout_18.addWidget(self.spinBoxOverlap)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_18)
        self.label_28 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_28.setObjectName("label_28")
        self.verticalLayout_2.addWidget(self.label_28)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_27 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_27.setObjectName("label_27")
        self.horizontalLayout_19.addWidget(self.label_27)
        self.spinBoxGnb = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxGnb.setMinimum(-1)
        self.spinBoxGnb.setMaximum(99)
        self.spinBoxGnb.setProperty("value", 0)
        self.spinBoxGnb.setObjectName("spinBoxGnb")
        self.horizontalLayout_19.addWidget(self.spinBoxGnb)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_19)
        self.label_30 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_30.setObjectName("label_30")
        self.verticalLayout_2.addWidget(self.label_30)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.label_29 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_29.setObjectName("label_29")
        self.horizontalLayout_20.addWidget(self.label_29)
        self.spinBoxNb_patch = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxNb_patch.setMinimum(-1)
        self.spinBoxNb_patch.setProperty("value", 0)
        self.spinBoxNb_patch.setObjectName("spinBoxNb_patch")
        self.horizontalLayout_20.addWidget(self.spinBoxNb_patch)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_20.addItem(spacerItem7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_20)
        self.label_32 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_32.setObjectName("label_32")
        self.verticalLayout_2.addWidget(self.label_32)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_31 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_31.setObjectName("label_31")
        self.horizontalLayout_21.addWidget(self.label_31)
        self.spinBoxK = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxK.setMinimum(0)
        self.spinBoxK.setProperty("value", 10)
        self.spinBoxK.setObjectName("spinBoxK")
        self.horizontalLayout_21.addWidget(self.spinBoxK)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_21.addItem(spacerItem8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_21)
        self.horizontalLayout_15.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_23 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_23.setObjectName("horizontalLayout_23")
        self.label_35 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_35.setObjectName("label_35")
        self.horizontalLayout_23.addWidget(self.label_35)
        self.spinBox_ssub = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_ssub.setMinimum(1)
        self.spinBox_ssub.setProperty("value", 1)
        self.spinBox_ssub.setObjectName("spinBox_ssub")
        self.horizontalLayout_23.addWidget(self.spinBox_ssub)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_23.addWidget(self.line)
        self.label_36 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_36.setObjectName("label_36")
        self.horizontalLayout_23.addWidget(self.label_36)
        self.spinBox_tsub = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_tsub.setMinimum(1)
        self.spinBox_tsub.setSingleStep(1)
        self.spinBox_tsub.setProperty("value", 1)
        self.spinBox_tsub.setObjectName("spinBox_tsub")
        self.horizontalLayout_23.addWidget(self.spinBox_tsub)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_23.addItem(spacerItem9)
        self.verticalLayout.addLayout(self.horizontalLayout_23)
        self.checkBox_low_rank_background = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBox_low_rank_background.setObjectName("checkBox_low_rank_background")
        self.verticalLayout.addWidget(self.checkBox_low_rank_background)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_4.addWidget(self.label_6)
        self.doubleSpinBox_ring_size_factor = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBox_ring_size_factor.setSingleStep(0.1)
        self.doubleSpinBox_ring_size_factor.setProperty("value", 1.5)
        self.doubleSpinBox_ring_size_factor.setObjectName("doubleSpinBox_ring_size_factor")
        self.horizontalLayout_4.addWidget(self.doubleSpinBox_ring_size_factor)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem10)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.comboBoxDeconv = QtWidgets.QComboBox(self.dockWidgetContents)
        self.comboBoxDeconv.setObjectName("comboBoxDeconv")
        self.comboBoxDeconv.addItem("")
        self.comboBoxDeconv.addItem("")
        self.comboBoxDeconv.addItem("")
        self.horizontalLayout_5.addWidget(self.comboBoxDeconv)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem11)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_22 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_22.setObjectName("label_22")
        self.horizontalLayout_14.addWidget(self.label_22)
        self.doubleSpinBoxMergeThresh = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxMergeThresh.setMaximum(1.0)
        self.doubleSpinBoxMergeThresh.setSingleStep(0.05)
        self.doubleSpinBoxMergeThresh.setProperty("value", 0.7)
        self.doubleSpinBoxMergeThresh.setObjectName("doubleSpinBoxMergeThresh")
        self.horizontalLayout_14.addWidget(self.doubleSpinBoxMergeThresh)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem12)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.groupBox_cnmf_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_cnmf_kwargs.setStatusTip("")
        self.groupBox_cnmf_kwargs.setCheckable(True)
        self.groupBox_cnmf_kwargs.setChecked(False)
        self.groupBox_cnmf_kwargs.setObjectName("groupBox_cnmf_kwargs")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_cnmf_kwargs)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.plainTextEdit_cnmf_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_cnmf_kwargs)
        self.plainTextEdit_cnmf_kwargs.setObjectName("plainTextEdit_cnmf_kwargs")
        self.verticalLayout_3.addWidget(self.plainTextEdit_cnmf_kwargs)
        self.verticalLayout.addWidget(self.groupBox_cnmf_kwargs)
        self.label_23 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setObjectName("label_23")
        self.verticalLayout.addWidget(self.label_23)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_12 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_8.addWidget(self.label_12)
        self.spinBoxMinSNR = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBoxMinSNR.setProperty("value", 1)
        self.spinBoxMinSNR.setObjectName("spinBoxMinSNR")
        self.horizontalLayout_8.addWidget(self.spinBoxMinSNR)
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem13)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.label_14 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_14.setObjectName("label_14")
        self.verticalLayout.addWidget(self.label_14)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_15 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_9.addWidget(self.label_15)
        self.doubleSpinBoxRValuesMin = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxRValuesMin.setMaximum(1.0)
        self.doubleSpinBoxRValuesMin.setSingleStep(0.025)
        self.doubleSpinBoxRValuesMin.setProperty("value", 0.7)
        self.doubleSpinBoxRValuesMin.setObjectName("doubleSpinBoxRValuesMin")
        self.horizontalLayout_9.addWidget(self.doubleSpinBoxRValuesMin)
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem14)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        self.label_17 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_17.setObjectName("label_17")
        self.verticalLayout.addWidget(self.label_17)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_16 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_10.addWidget(self.label_16)
        self.doubleSpinBoxDecayTime = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxDecayTime.setMinimum(0.1)
        self.doubleSpinBoxDecayTime.setSingleStep(0.5)
        self.doubleSpinBoxDecayTime.setProperty("value", 4.0)
        self.doubleSpinBoxDecayTime.setObjectName("doubleSpinBoxDecayTime")
        self.horizontalLayout_10.addWidget(self.doubleSpinBoxDecayTime)
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem15)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.groupBox_eval_kwargs = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_eval_kwargs.setCheckable(True)
        self.groupBox_eval_kwargs.setChecked(False)
        self.groupBox_eval_kwargs.setObjectName("groupBox_eval_kwargs")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_eval_kwargs)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.plainTextEdit_eval_kwargs = QtWidgets.QPlainTextEdit(self.groupBox_eval_kwargs)
        self.plainTextEdit_eval_kwargs.setObjectName("plainTextEdit_eval_kwargs")
        self.verticalLayout_5.addWidget(self.plainTextEdit_eval_kwargs)
        self.verticalLayout.addWidget(self.groupBox_eval_kwargs)
        self.checkBoxKeepMemmap = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxKeepMemmap.setObjectName("checkBoxKeepMemmap")
        self.verticalLayout.addWidget(self.checkBoxKeepMemmap)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_18 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_11.addWidget(self.label_18)
        self.lineEdName = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.lineEdName.setObjectName("lineEdName")
        self.horizontalLayout_11.addWidget(self.lineEdName)
        self.btnAddToBatchCNMFE = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnAddToBatchCNMFE.setObjectName("btnAddToBatchCNMFE")
        self.horizontalLayout_11.addWidget(self.btnAddToBatchCNMFE)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_15.addLayout(self.verticalLayout)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        self.lineEdName.returnPressed.connect(self.btnAddToBatchCNMFE.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.spinBoxGSig, self.lineEdCorrPNRName)
        DockWidget.setTabOrder(self.lineEdCorrPNRName, self.btnAddToBatchCorrPNR)
        DockWidget.setTabOrder(self.btnAddToBatchCorrPNR, self.lineEditAin)
        DockWidget.setTabOrder(self.lineEditAin, self.spinBox_p)
        DockWidget.setTabOrder(self.spinBox_p, self.doubleSpinBoxMinCorr)
        DockWidget.setTabOrder(self.doubleSpinBoxMinCorr, self.spinBoxMinPNR)
        DockWidget.setTabOrder(self.spinBoxMinPNR, self.spinBoxRf)
        DockWidget.setTabOrder(self.spinBoxRf, self.spinBoxOverlap)
        DockWidget.setTabOrder(self.spinBoxOverlap, self.spinBoxGnb)
        DockWidget.setTabOrder(self.spinBoxGnb, self.spinBoxNb_patch)
        DockWidget.setTabOrder(self.spinBoxNb_patch, self.spinBoxK)
        DockWidget.setTabOrder(self.spinBoxK, self.spinBox_ssub)
        DockWidget.setTabOrder(self.spinBox_ssub, self.spinBox_tsub)
        DockWidget.setTabOrder(self.spinBox_tsub, self.checkBox_low_rank_background)
        DockWidget.setTabOrder(self.checkBox_low_rank_background, self.doubleSpinBox_ring_size_factor)
        DockWidget.setTabOrder(self.doubleSpinBox_ring_size_factor, self.comboBoxDeconv)
        DockWidget.setTabOrder(self.comboBoxDeconv, self.doubleSpinBoxMergeThresh)
        DockWidget.setTabOrder(self.doubleSpinBoxMergeThresh, self.groupBox_cnmf_kwargs)
        DockWidget.setTabOrder(self.groupBox_cnmf_kwargs, self.plainTextEdit_cnmf_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_cnmf_kwargs, self.spinBoxMinSNR)
        DockWidget.setTabOrder(self.spinBoxMinSNR, self.doubleSpinBoxRValuesMin)
        DockWidget.setTabOrder(self.doubleSpinBoxRValuesMin, self.doubleSpinBoxDecayTime)
        DockWidget.setTabOrder(self.doubleSpinBoxDecayTime, self.groupBox_eval_kwargs)
        DockWidget.setTabOrder(self.groupBox_eval_kwargs, self.plainTextEdit_eval_kwargs)
        DockWidget.setTabOrder(self.plainTextEdit_eval_kwargs, self.checkBoxKeepMemmap)
        DockWidget.setTabOrder(self.checkBoxKeepMemmap, self.lineEdName)
        DockWidget.setTabOrder(self.lineEdName, self.btnAddToBatchCNMFE)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "&CNMF-E"))
        self.label_5.setToolTip(_translate("DockWidget", "Compute some summary images (correlation and peak to noise ratio)"))
        self.label_5.setText(_translate("DockWidget", "Inspect Correlation and PNR"))
        self.label_2.setText(_translate("DockWidget", "gaussian width of a 2D gaussian kernel, which approximates a neuron\n"
"3 times less than the average diamters of a neuron (pixels)"))
        self.label.setText(_translate("DockWidget", "gSig:"))
        self.label_9.setText(_translate("DockWidget", "Stop here:"))
        self.lineEdCorrPNRName.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchCorrPNR.setText(_translate("DockWidget", "Add to batch"))
        self.label_20.setText(_translate("DockWidget", "CNMF-E"))
        self.label_3.setText(_translate("DockWidget", "Ain:"))
        self.label_19.setText(_translate("DockWidget", "p:"))
        self.label_7.setText(_translate("DockWidget", "Minimum correlation of peak"))
        self.label_8.setText(_translate("DockWidget", "min_corr:"))
        self.label_11.setText(_translate("DockWidget", "Minimum peak to noise ratio"))
        self.label_10.setText(_translate("DockWidget", "min_pnr:"))
        self.label_13.setText(_translate("DockWidget", "Adaptive way to set threshold on the transient size"))
        self.label_24.setText(_translate("DockWidget", "Half size of patch"))
        self.label_21.setText(_translate("DockWidget", "rf:"))
        self.label_25.setText(_translate("DockWidget", "Overlap of patches (at least 4 times the size of a neuron/cell)"))
        self.label_26.setText(_translate("DockWidget", "overlap:"))
        self.label_28.setText(_translate("DockWidget", "Global number of background components"))
        self.label_27.setText(_translate("DockWidget", "gnb:"))
        self.label_30.setText(_translate("DockWidget", "Background components per patch"))
        self.label_29.setText(_translate("DockWidget", "nb_patch:"))
        self.label_32.setText(_translate("DockWidget", "Number of neurons/cell per patch"))
        self.label_31.setText(_translate("DockWidget", "k:"))
        self.label_35.setText(_translate("DockWidget", "ssub:"))
        self.label_36.setText(_translate("DockWidget", "tsub:"))
        self.checkBox_low_rank_background.setText(_translate("DockWidget", "low_rank_background"))
        self.label_6.setText(_translate("DockWidget", "ring_size_factor:"))
        self.label_4.setText(_translate("DockWidget", "deconvolution: "))
        self.comboBoxDeconv.setItemText(0, _translate("DockWidget", "oasis"))
        self.comboBoxDeconv.setItemText(1, _translate("DockWidget", "cvxpy"))
        self.comboBoxDeconv.setItemText(2, _translate("DockWidget", "SKIP"))
        self.label_22.setText(_translate("DockWidget", "merge_thresh:"))
        self.groupBox_cnmf_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.groupBox_cnmf_kwargs.setTitle(_translate("DockWidget", "Use CNMF &kwargs"))
        self.plainTextEdit_cnmf_kwargs.setToolTip(_translate("DockWidget", "You can enter additional kwargs to pass for CNMF instantiation.\n"
"Use single quotes for strings, do not use double quotes."))
        self.label_23.setText(_translate("DockWidget", "Evalulation params"))
        self.label_12.setText(_translate("DockWidget", "min_SNR:"))
        self.label_14.setText(_translate("DockWidget", "Threshold on space consistency \n"
"If lower more components will be accepted, potentially with worse quality"))
        self.label_15.setText(_translate("DockWidget", "r_values_min:"))
        self.label_17.setText(_translate("DockWidget", "Average decay time of calcium spikes (seconds)"))
        self.label_16.setText(_translate("DockWidget", "decay_time:"))
        self.groupBox_eval_kwargs.setToolTip(_translate("DockWidget", "You can enter additional parameters to use for component evaluation.\n"
"Use single quotes for strings, do not use double quotes"))
        self.groupBox_eval_kwargs.setTitle(_translate("DockWidget", "Use e&valuation params"))
        self.plainTextEdit_eval_kwargs.setToolTip(_translate("DockWidget", "You can enter additional parameters to use for component evaluation.\n"
"Use single quotes for strings, do not use double quotes"))
        self.checkBoxKeepMemmap.setText(_translate("DockWidget", "Keep memmap"))
        self.label_18.setText(_translate("DockWidget", "Perform CNMF-E:"))
        self.lineEdName.setPlaceholderText(_translate("DockWidget", "Enter name"))
        self.btnAddToBatchCNMFE.setText(_translate("DockWidget", "Add to batch"))





# -*- coding: utf-8 -*-
"""
Created on July 7 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

Adapted from @agiovann and @epnev
"""

from __future__ import division
import sys
import cv2
from glob import glob
try:
    cv2.setNumThreads(1)
except:
    pass
import numpy as np
import os
from time import time
import caiman as cm
from caiman.source_extraction.cnmf import cnmf as cnmf
from caiman.utils.utils import load_dict_from_hdf5
from caiman.source_extraction.cnmf.params import CNMFParams
import pickle
import traceback
import json
import logging
import tifffile

# must be within this block, else windows gives issues
if __name__ == '__main__':
    from mesmerize.common.utils import HdfTools

if not sys.argv[0] == __file__:
    from ..roi_manager import ModuleGUI
    from ...core import ViewerUtils, ViewerWorkEnv


def run(batch_dir: str, UUID: str):
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG,
                        format="%(relativeCreated)12d [%(filename)s:%(funcName)20s():%(lineno)s] [%(process)d] %(message)s")
    start_time = time()

    output = {'status': 0, 'output_info': ''}

    file_path = os.path.join(batch_dir, UUID)
    input_params = pickle.load(open(file_path + '.params', 'rb'))

    try:
        if 'is_3d' in input_params.keys():
            if input_params['is_3d']:
                run_multi(batch_dir, UUID, output)
            else:
                run_single(batch_dir, UUID, output)
        else:
            run_single(batch_dir, UUID, output)

    except Exception as e:
        output.update({'status': 0, 'output_info': traceback.format_exc()})

    if not input_params.get('keep_memmap', False):
        for mf in glob(os.path.join(batch_dir, f'memmap-{UUID}*')):
            try:
                os.remove(mf)
            except:
                pass

    end_time = time()
    proc_time = (end_time - start_time) / 60

    output.update({'processing_time': proc_time})

    json.dump(output, open(file_path + '.out', 'w'))


def run_single(batch_dir, UUID, output):
    n_processes = os.environ['_MESMERIZE_N_THREADS']
    n_processes = int(n_processes)
    file_path = os.path.join(batch_dir, UUID)

    filename = [file_path + '_input.tiff']
    input_params = pickle.load(open(file_path + '.params', 'rb'))

    print('*********** Creating Process Pool ***********')
    c, dview, n_processes = cm.cluster.setup_cluster(
        backend='local', n_processes=n_processes, single_thread=False, ignore_preexisting=True
    )

    memmap_fname = cm.save_memmap(
        filename,
        base_name=f'memmap-{UUID}',
        order='C',
        border_to_0=input_params['border_pix'],
        dview=dview
    )

    Yr, dims, T = cm.load_memmap(memmap_fname)
    Y = np.reshape(Yr.T, [T] + list(dims), order='F')

    Ain = None

    # seed components
    if 'use_seeds' in input_params.keys():
        if input_params['use_seeds']:
            try:
                # see if it's an h5 file produced by the nuset_segment GUI
                hdict = HdfTools.load_dict(
                    os.path.join(f'{file_path}.ain'),
                    'data'
                )
                Ain = hdict['sparse_mask']
            except:
                try:
                    Ain = np.load(f'{file_path}.ain')
                except Exception as e:
                    output['warnings'] = f'Could not seed components, make sure that ' \
                        f'the .ain file exists in the batch dir: {e}'

    # seeded
    if Ain is not None:
        input_params['cnmf_kwargs'].update(
            {
                'only_init': False,
                'rf': None
            }
        )

    cnmf_params = CNMFParams(params_dict=input_params['cnmf_kwargs'])

    cnm = cnmf.CNMF(
        dview=dview,
        n_processes=n_processes,
        Ain=Ain,
        params=cnmf_params,
    )

    cnm.fit(Y)

    if input_params['refit']:
        cnm = cnm.refit(Y, dview=dview)

    cnm.params.change_params(params_dict=input_params['eval_kwargs'])

    cnm.estimates.evaluate_components(
        Y,
        cnm.params,
        dview=dview
    )

    cnm.estimates.select_components(use_object=True)

    out_filename = f'{UUID}_results.hdf5'
    cnm.save(out_filename)

    output_files = [out_filename]

    output.update(
        {
            'output': UUID,
            'status': 1,
            'output_files': output_files
        }
    )

    dview.terminate()

    return output


def run_multi(batch_dir, UUID, output):
    n_processes = os.environ['_MESMERIZE_N_THREADS']
    n_processes = int(n_processes)
    file_path = os.path.join(batch_dir, UUID)

    filename = [file_path + '_input.tiff']
    input_params = pickle.load(open(file_path + '.params', 'rb'))

    seq = tifffile.TiffFile(filename[0]).asarray()
    seq_shape = seq.shape

    # assume default tzxy
    for z in range(seq.shape[1]):
        tifffile.imsave(f'{file_path}_z{z}.tiff', seq[:, z, :, :])

    del seq

    print('*********** Creating Process Pool ***********')
    c, dview, n_processes = cm.cluster.setup_cluster(
        backend='local', n_processes=n_processes, single_thread=False, ignore_preexisting=True
    )
    num_components = 0
    output_files = []
    for z in range(seq_shape[1]):
        print(f"Plane {z} / {seq_shape[1]}")
        filename = [f'{file_path}_z{z}.tiff']
        print('Creating memmap')

        memmap_fname = cm.save_memmap(
            filename,
            base_name=f'memmap-{UUID}',
            order='C',
            border_to_0=input_params['border_pix'],
            dview=dview
        )

        Yr, dims, T = cm.load_memmap(memmap_fname)
        Y = np.reshape(Yr.T, [T] + list(dims), order='F')

        Ain = None

        # seed components
        # see if it's an h5 file produced by the nuset_segment GUI
        try:
            hdict = HdfTools.load_dict(os.path.join(f'{file_path}.ain'), 'data')
            Ain = hdict[f'sparse_mask'][str(z)]
        except Exception as e:
            output['warnings'] = f'Could not seed components, make sure that ' \
                f'the .ain file exists in the batch dir: {e}'

        #print(Ain)
        #raise Exception

        # seeded
        if Ain is not None:
            input_params['cnmf_kwargs'].update(
                {
                    'only_init': False,
                    'rf': None
                }
            )

        cnmf_params = CNMFParams(params_dict=input_params['cnmf_kwargs'])

        cnm = cnmf.CNMF(
            dview=dview,
            n_processes=n_processes,
            Ain=Ain,
            params=cnmf_params,
        )

        cnm.fit(Y)

        if input_params['refit']:
            cnm = cnm.refit(Y, dview=dview)

        cnm.params.set('quality', input_params['eval_kwargs'])

        cnm.estimates.evaluate_components(
            Y,
            cnm.params,
            dview=dview
        )

        cnm.estimates.select_components(use_object=True)

        num_components += len(cnm.estimates.C)

        out_filename = f'{UUID}_results_z{z}.hdf5'
        cnm.save(out_filename)

        output_files.append(out_filename)

        os.remove(filename[0])

    output.update(
        {
            'output': UUID,
            'status': 1,
            'output_files': output_files,
            'num_components': num_components
        }
    )

    dview.terminate()

    return output


class Output:
    def __init__(self, batch_path, UUID, viewer_ref):
        vi = ViewerUtils(viewer_ref)

        if not vi.discard_workEnv():
            return

        vi.viewer.status_bar_label.showMessage('Please wait, loading image data...')

        pik_path = os.path.join(batch_path, f'{UUID}_input.pik')
        tiff_path = os.path.join(batch_path, f'{UUID}_input.tiff')
        vi.viewer.workEnv = ViewerWorkEnv.from_pickle(pik_path, tiff_path)

        vi.update_workEnv()
        vi.viewer.status_bar_label.showMessage('Finished loading image data...')

        input_params_path = os.path.join(batch_path, f'{UUID}.params')
        input_params = pickle.load(open(input_params_path, 'rb'))

        if input_params['is_3d']:
            roi_manager_gui = vi.viewer.parent().get_module('roi_manager')
            manager = roi_manager_gui.start_backend('VolMultiCNMFROI')

            cnmf_data_dicts = \
                [
                    load_dict_from_hdf5(
                        os.path.join(batch_path, f'{UUID}_results_z{z}.hdf5')
                    )
                    # for z in range(5)
                    for z in range(vi.viewer.workEnv.imgdata.z_max + 1)
                ]

            manager.add_all_components(
                cnmf_data_dicts,
                input_params
            )
        else:
            cnmf_data = load_dict_from_hdf5(
                os.path.join(batch_path, f'{UUID}_results.hdf5')
            )

            roi_manager_gui = vi.viewer.parent().get_module('roi_manager')
            roi_manager_gui.start_backend('CNMFROI')

            roi_manager_gui.manager.add_all_components(
                cnmf_data,
                input_params_dict=input_params,
            )

        name = input_params['item_name']
        vi.viewer.ui.label_curr_img_seq_name.setText(f'CNMF: {name}')
        vi.viewer.workEnv.history_trace.append({'cnmf': input_params})
        vi.enable_ui(True)


#if sys.argv[0] == __file__:
if __name__ == '__main__':
    run(sys.argv[1], sys.argv[2])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2019 Kushal Kolar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from fcsugar import *
from scipy.stats import zscore as _zscore
import numpy as np
from ..containers import TransmissionContainer


@node
def zscore(container: TransmissionContainer, data_column: str, axis: int = None):
    container.df['zscore'] = container.df[data_column].apply(_zscore)
    return container



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 14 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from PyQt5 import QtWidgets
from ..plot_window import PlotWindow
from ...variants import PgCurvePlot
from numpy import ndarray


class CurvePlotWindow(PlotWindow):
    def __init__(self, parent=None):
        super(CurvePlotWindow, self).__init__(parent)
        self.setWindowTitle('Curves Plot Window')

        self.ui.comboBoxUUIDColumn.setDisabled(True)

        self.add_plot_tab('Curves')
        self.curve_plot = PgCurvePlot(self.graphicsViews['Curves'])
        self.legends = []

        self.plotting_tabs = {'Curves': self.curve_plot}

    def update_params(self):
        super(CurvePlotWindow, self).update_params()
        self.update_curve_plot()

    def update_curve_plot(self):
        for tab_ix in range(1, self.ui.tabWidget.count()):
            self.ui.tabWidget.removeTab(1)

        self.curve_plot.clear_plot()
        colors = self.auto_colormap(len(self.groups))
        errors = []

        accepted_datatypes = [ndarray]

        if not self.data_types_check(accepted_datatypes):
            return

        for plot_ix, data_column in enumerate(self.data_columns):

            msg = 'Progress: ' + \
                  str(((plot_ix + 1) / len(self.data_columns)) * 100) + ' % , Plotting data column: ' + data_column
            self.status_bar.showMessage(msg)

            self.curve_plot.add_plot(data_column)
            self.curve_plot.create_legend(plot_ix)

            for ii, (dataframe, group) in enumerate(zip(self.group_dataframes, self.groups)):
                self.status_bar.showMessage(msg + ', plotting group: ' + group)

                self.add_plot_tab(title=group)
                group_plot = PgCurvePlot(self.graphicsViews[group])
                self.plotting_tabs.update({group: group_plot})

                self.curve_plot.add_data_to_plot(plot_ix, data_series=dataframe[data_column],
                                                 name=group, color=colors[ii])

                rng = self.curve_plot.get_range(plot_ix)
                self.curve_plot.set_range(plot_ix, x_range=rng[0], y_range=rng[1])

                group_plot.add_plot(data_column)
                group_plot.add_data_to_plot(plot_ix, data_series=dataframe[data_column],
                                            name=group, color=colors[ii])
                group_plot.set_range(plot_ix, x_range=rng[0], y_range=rng[1])

        if len(errors) > 0:
            QtWidgets.QMessageBox.warning(self, 'Errors while plotting',
                                          'The following errors occured while plotting:\n' + '\n'.join(errors))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on April 21 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from .main_window_pytemplate import *
from ..pyqtgraphCore.Qt import QtCore, QtGui, QtWidgets
from ..pyqtgraphCore.console import ConsoleWidget
from ..pyqtgraphCore.imageview import ImageView
from ..pyqtgraphCore import setConfigOptions
from .modules import *
from .core.common import ViewerUtils
import numpy as np
from .core.viewer_work_environment import ViewerWorkEnv
from .core.data_types import ImgData
from ..common import configuration, doc_pages
from ..common import get_window_manager
from .image_menu.main import ImageMenu
from spyder.widgets.variableexplorer import objecteditor
import traceback
from .core.add_to_project import AddToProjectDialog
import os
from . import image_utils
import importlib
import importlib.util
# from .modules import custom_modules
from functools import partial
import sys
import pandas as pd
from typing import Optional, Union
from uuid import UUID as UUID_type
import logging


logger = logging.getLogger()

_custom_modules_dir = configuration.get_sys_config()['_MESMERIZE_CUSTOM_MODULES_DIR']
_custom_modules_package_name = os.path.basename(_custom_modules_dir)
_cmi = os.path.join(_custom_modules_dir, '__init__.py')

if os.path.isfile(_cmi):
    _parent_dir = os.path.abspath(os.path.join(_custom_modules_dir, os.path.pardir))
    sys.path.append(_parent_dir)
    _spec = importlib.util.spec_from_file_location('custom_modules', _cmi)
    custom_modules = importlib.util.module_from_spec(_spec)
    _spec.loader.exec_module(custom_modules)
    _import_custom_modules = True
else:
    _import_custom_modules = False


dock_widget_area = \
    {
        'roi_manager': QtCore.Qt.LeftDockWidgetArea,
        roi_manager.ModuleGUI: QtCore.Qt.LeftDockWidgetArea,

        'tiff_io': QtCore.Qt.TopDockWidgetArea,
        tiff_io.ModuleGUI: QtCore.Qt.TopDockWidgetArea,

        'suite2p_importer': QtCore.Qt.TopDockWidgetArea,
        suite2p.ModuleGUI: QtCore.Qt.TopDockWidgetArea,

        'stimulus_mapping': QtCore.Qt.BottomDockWidgetArea,
        stimulus_mapping.ModuleGUI: QtCore.Qt.BottomDockWidgetArea,
    }

# inscopix importer
if sys.version_info[1] >= 8:
    dock_widget_area.update(
        {
            'inscopix_importer': QtCore.Qt.TopDockWidgetArea,
            inscopix_importer.ModuleGUI: QtCore.Qt.TopDockWidgetArea,
        }
)


# caiman modules
if configuration.HAS_CAIMAN:
    dock_widget_area.update(
        {
            'cnmf': QtCore.Qt.RightDockWidgetArea,
            cnmf.ModuleGUI: QtCore.Qt.RightDockWidgetArea,

            'cnmfe': QtCore.Qt.RightDockWidgetArea,
            cnmfe.ModuleGUI: QtCore.Qt.RightDockWidgetArea,

            'cnmf_3d': QtCore.Qt.RightDockWidgetArea,
            cnmf_3d.ModuleGUI: QtCore.Qt.RightDockWidgetArea,

            'caiman_motion_correction': QtCore.Qt.RightDockWidgetArea,
            caiman_motion_correction.ModuleGUI: QtCore.Qt.RightDockWidgetArea,

            'caiman_importer': QtCore.Qt.TopDockWidgetArea,
            caiman_importer.ModuleGUI: QtCore.Qt.TopDockWidgetArea,

            'caiman_dfof': QtCore.Qt.RightDockWidgetArea,
            caiman_dfof.ModuleGUI: QtCore.Qt.RightDockWidgetArea,
        }
    )


file_associations =\
    {
        '.tif':     tiff_io.ModuleGUI,
        '.tiff':    tiff_io.ModuleGUI,
        '.btf':     tiff_io.ModuleGUI,
    }


class MainWindow(QtWidgets.QMainWindow):
    standard_modules = {'tiff_io': tiff_io.ModuleGUI,
                        'mesfile': mesfile_io.ModuleGUI,
                        'roi_manager': roi_manager.ModuleGUI,
                        'suite2p_importer': suite2p.ModuleGUI,
                        'stimulus_mapping': stimulus_mapping.ModuleGUI,
                        'script_editor': script_editor.ModuleGUI,
                        'mesc_importer': femtonics_mesc.ModuleGUI,
                        'exporter': exporter.ModuleGUI,
                        }

    if configuration.HAS_TENSORFLOW:
        standard_modules.update(
            {'nuset_segment': nuset_segment.ModuleGUI}
        )

    # caiman modules
    if configuration.HAS_CAIMAN:
        standard_modules.update(
            {
                'cnmf': cnmf.ModuleGUI,
                'cnmfe': cnmfe.ModuleGUI,
                'cnmf_3d': cnmf_3d.ModuleGUI,
                'caiman_motion_correction': caiman_motion_correction.ModuleGUI,
                'caiman_importer': caiman_importer.ModuleGUI,
                'caiman_dfof': caiman_dfof.ModuleGUI,
            }
        )

    available_modules = list(standard_modules.keys())

    def __init__(self, parent=None):
        QtWidgets.QMainWindow.__init__(self, parent=parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.running_modules = []
        # TODO: Integrate viewer initiation here instead of outside
        self.ui.action_mes_importer.triggered.connect(lambda: self.run_module(mesfile_io.ModuleGUI))
        self.ui.action_mesc_importer.triggered.connect(lambda: self.run_module(femtonics_mesc.ModuleGUI))
        self.ui.actionTiff_file.triggered.connect(lambda: self.run_module(tiff_io.ModuleGUI))
        self.ui.actionROI_Manager.triggered.connect(lambda: self.run_module(roi_manager.ModuleGUI))
        self.ui.actionSuite2p_Importer.triggered.connect(lambda: self.run_module(suite2p.ModuleGUI))
        self.ui.actionStimulus_Mapping.triggered.connect(lambda: self.run_module(stimulus_mapping.ModuleGUI))
        self.ui.actionScript_Editor.triggered.connect(lambda: self.run_module(script_editor.ModuleGUI))

        if sys.version_info[1] >= 8:
            self.ui.actionInscopix_isxd.triggered.connect(lambda: self.run_module(inscopix_importer.ModuleGUI))

        if configuration.HAS_TENSORFLOW:
            self.ui.actionNuSeT_Segmentation.triggered.connect(lambda: self.run_module(nuset_segment.ModuleGUI))

        # TODO: refactor the actions trigger connections so they're automated based on module name
        if configuration.HAS_CAIMAN:
            self.ui.actionCNMF.triggered.connect(lambda: self.run_module(cnmf.ModuleGUI))
            self.ui.actionCNMF_E.triggered.connect(lambda: self.run_module(cnmfe.ModuleGUI))
            self.ui.actionCNMF_3D.triggered.connect(lambda: self.run_module(cnmf_3d.ModuleGUI))
            self.ui.actionMotion_Correction.triggered.connect(
                lambda: self.run_module(caiman_motion_correction.ModuleGUI)
            )
            self.ui.actionCaimanImportHDF5.triggered.connect(lambda: self.run_module(caiman_importer.ModuleGUI))
            self.ui.actionCaimanDFOF.triggered.connect(lambda: self.run_module(caiman_dfof.ModuleGUI))

        self.ui.actionWork_Environment_Info.triggered.connect(self.open_workEnv_editor)
        self.ui.actionAdd_to_project.triggered.connect(self.add_work_env_to_project)
        self.ui.actionSave_work_environment.triggered.connect(self.save_work_environment_dialog)
        self.ui.actionOpen_docs.triggered.connect(doc_pages['viewer'])
        self.ui.actionReport_issue_bug.triggered.connect(doc_pages['issue-tracker'])
        self.ui.actionQuestions_discussion.triggered.connect(doc_pages['gitter'])

        self.add_to_project_dialog = None
        self.custom_modules = None
        self._cms = []
        self._cms_actions = []

        self.setAcceptDrops(True)

        if _import_custom_modules:
            self.set_custom_module_triggers()

        self.ui.dockConsole.hide()

    def set_custom_module_triggers(self):
        self.custom_modules = dict()

        failed_imports = []
        for mstr in custom_modules.__all__:
            try:
                mstr = '.' + mstr
                mod = importlib.import_module(mstr, package=_custom_modules_package_name)
                c = getattr(mod, 'ModuleGUI')
                self.custom_modules[mod.module_name] = c

                name = mod.module_name
                action = QtWidgets.QAction(self)
                action.setCheckable(False)
                action.setObjectName("custom_module" + name)
                action.setText(name)

                action.triggered.connect(partial(self.run_module, c))

                self._cms_actions.append(action)
                self.ui.menuCustom_Modules.addAction(self._cms_actions[-1])
            except ImportError:
                failed_imports.append(
                    (mstr[1:], traceback.format_exc())
                )

        if len(failed_imports) > 0:
            names = '\n'.join(fi[0] for fi in failed_imports)
            QtWidgets.QMessageBox.warning(
                self,
                'Failed to load plugings',
                f'The following plugins failed to load:\n{names}'
                f'\n\n'
                f'See the terminal for detailed tracebacks'
            )

            for fi in failed_imports:
                print(
                    f'{fi[0]}'
                    f'\n{fi[1]}'
                    f'\n\n'
                )

        self.available_modules += list(self.custom_modules.keys())

    @property
    def viewer_reference(self):
        return self._viewer

    @viewer_reference.setter
    def viewer_reference(self, viewer: ImageView):
        self._viewer = viewer
        self._viewer.workEnv = ViewerWorkEnv()

        self.run_module(roi_manager.ModuleGUI, hide=True)
        self.roi_manager = self.running_modules[-1]

        self._viewer.ui.btnExportWorkEnv.clicked.connect(
            partial(self.run_module, 'exporter')
        )

        status_label = self.statusBar()

        self._viewer.status_bar_label = status_label

        self.initialize_menubar_triggers()

        ns = {'np':                 np,
              'vi':                 self.vi,
              'viewer':             self.vi.viewer,
              'ViewerWorkEnv':      ViewerWorkEnv,
              'ImgData':            ImgData,
              'get_workEnv':        self.vi.viewer.get_workEnv,
              'get_image':          lambda: self.vi.viewer.get_workEnv().imgdata.seq,
              'get_meta':           lambda: self.vi.viewer.get_workEnv().imgdata.meta,
              'update_workEnv':     self.vi.update_workEnv,
              'clear_workEnv':      self.vi._clear_workEnv,
              'get_module':         self.get_module,
              'get_batch_manager':  self.get_batch_manager,
              'all_modules':        self.available_modules,
              'image_utils':        image_utils,
              'this': self
              }

        txt = "Namespaces:          \n" \
              "numpy as np          \n" \
              "ViewerUtils as vi \n" \
              "self as this \n" \
              "ViewerWorkEnv and ImgData for factory purposes\n" \
              "useful callables for scripting, see docs for examples:\n" \
              "get_workEnv(), get_image(), get_meta(), get_module(<module name>), get_batch_manager()\n" \
              "update_workEnv(), clear_workEnv()\n" \
              "'all_modules' will list all available modules\n" \
              "Some basic image utility functions: image_utils"

        cmd_history_file = os.path.join(configuration.console_history_path, 'viewer.pik')

        self.console = ConsoleWidget(namespace=ns, text=txt, historyFile=cmd_history_file)

        self.ui.dockConsole.setWidget(self.console)

    def run_module(self, module_class, hide=False) -> object:
        """
        :param module_class: The python module imported within this main_window module or the module name as a str
        :param hide: To not show the widget, useful for scripting
        :return: Return the chosen module, instantiate it if it's not already running.
        """
        # Show the QDockableWidget if it's already running
        if type(module_class) is str:
            if self.custom_modules is None:
                cms = {}
            else:
                cms = self.custom_modules
            module_class = {**self.standard_modules, **cms}[module_class]

        for m in self.running_modules:
            if isinstance(m, module_class):
                if not hide:
                    m.show()
                return m

        # Else create instance and start running it
        m = module_class(self, self._viewer)
        self.running_modules.append(m)

        if not hide:
            self.running_modules[-1].show()
        else:
            self.running_modules[-1].hide()

        if (module_class in self.standard_modules.values()) and (module_class in dock_widget_area.keys()):
            self.addDockWidget(dock_widget_area[module_class], m)

        return self.running_modules[-1]

    def get_module(self, module_name: str, hide=False):
        return self.run_module(module_name, hide)

    def update_available_inputs(self):
        for m in self.running_modules:
            if hasattr(m, 'update_available_inputs'):
                m.update_available_inputs()

    def initialize_menubar_triggers(self):
        self.ui.actionBatch_Manager.triggered.connect(self.start_batch_manager)

        self.vi = ViewerUtils(self._viewer)

        self.ui.actionDump_Work_Environment.triggered.connect(self.vi.discard_workEnv)

        self.image_menu = ImageMenu(self.vi)

        self.ui.actionReset_Scale.triggered.connect(self.image_menu.reset_scale)
        self.ui.actionMeasure.triggered.connect(self.image_menu.action_measure_line_invoked)
        self.ui.actionResize.triggered.connect(self.image_menu.resize)
        self.ui.actionCrop.triggered.connect(self.image_menu.crop)

        self.ui.actionMean.triggered.connect(self.image_menu.mean_projection)
        self.ui.actionMax.triggered.connect(self.image_menu.max_projection)
        self.ui.actionStandard_Deviation.triggered.connect(self.image_menu.std_projection)
        self.ui.actionClose_all_projection_windows.triggered.connect(self.image_menu.close_projection_windows)

    def get_batch_manager(self) -> QtWidgets.QWidget:
        return get_window_manager().get_batch_manager()

    def start_batch_manager(self):
        batch_manager = self.get_batch_manager()
        batch_manager.show()

    def open_workEnv_editor(self):
        self.vi.viewer.status_bar_label.showMessage('Please wait, loading editor interface...')

        # if hasattr(self.vi.viewer.workEnv.roi_manager, 'roi_list'):
        #     roi_list = self.vi.viewer.workEnv.roi_manager.roi_list
        # else:
        #     roi_list = None

        # d = {'custom_columns_dict': self.vi.viewer.workEnv.custom_columns_dict,
        #      'isEmpty':             self.vi.viewer.workEnv.isEmpty,
        #      'imgdata':             self.vi.viewer.workEnv.imgdata.seq,
        #      'meta_data':           self.vi.viewer.workEnv.imgdata.meta,
        #      'roi_list':            roi_list,
        #      'comments':            self.vi.viewer.workEnv.comments,
        #      'origin_file':         self.vi.viewer.workEnv.origin_file,
        #      '_saved':              self.vi.viewer.workEnv._saved
        #      }

        try:
            changes = objecteditor.oedit(self.vi.viewer.workEnv)
        except:
            logger.info((traceback.format_exc()))
            # QtWidgets.QMessageBox.de(self, 'Unable to open work environment editor',
            #                               'The following error occured while trying to open the work environment editor:'
            #                               '\n', traceback.format_exc())
            return

        if changes is not None:
            try:
                    self.vi.viewer.workEnv = changes
                    self.vi.update_workEnv()
            except:
                QtWidgets.QMessageBox.warning(self, 'Unable to apply changes',
                                              'The following error occured while trying to save changes to the work '
                                              'environment. You may want to use the console and make sure your work '
                                              'environment has not been corrupted.'
                                              '\n' + str(traceback.format_exc()))
                return
        elif changes is None:
            self.vi.viewer.status_bar_label.showMessage('Work environment unchanged')
            return

        self.vi.viewer.status_bar_label.showMessage('Your edits were successfully applied to the work environment!')

        # You can even let the user save changes if they click "OK", and the function returns None if they cancel

    def add_work_env_to_project(self):
        if self.vi.viewer.workEnv.isEmpty:
            QtWidgets.QMessageBox.information(self, 'Empty work environment',
                                              'Nothing to save, work environment is empty')
            return
        if self.add_to_project_dialog is not None:
            try:
                self._delete_add_to_project_dialog()
                self.add_to_project_dialog = None
            except:
                pass

        if configuration.proj_path is None:
            QtWidgets.QMessageBox.question(self, 'No project open', 'You must have a project open to add to it')
            return

        if len(self.viewer_reference.workEnv.roi_manager.roi_list) == 0:
            QtWidgets.QMessageBox.warning(self, 'No curves', 'You do not have any curves in your work environment')
            return

        else:
            self.add_to_project_dialog = AddToProjectDialog(self.viewer_reference.workEnv)
            self.add_to_project_dialog.signal_finished.connect(self._delete_add_to_project_dialog)
            self.add_to_project_dialog.show()

    def _delete_add_to_project_dialog(self):
        self.add_to_project_dialog.deleteLater()

    def save_work_environment_dialog(self):
        path = QtWidgets.QFileDialog.getSaveFileName(self, 'Work environment file name')
        if path == '':
            return

        try:
            dirname = os.path.dirname(path[0])
            filename = os.path.basename(path[0])
            self.vi.viewer.workEnv.to_pickle(dir_path=dirname, filename=filename)
        except Exception:
            QtWidgets.QMessageBox.warning(self, 'File save Error', 'Unable to save the file\n' + traceback.format_exc())

    def open_work_environment_dialog(self):
        pass

    def closeEvent(self, QCloseEvent):
        if not self.vi.discard_workEnv():
            QCloseEvent.ignore()
        else:
            self.vi._clear_workEnv()
            QCloseEvent.accept()

    def open_from_dataframe(self, proj_path: str,
                            df: pd.DataFrame = None,
                            row: pd.Series = None,
                            sample_id: str = None,
                            uuid_curve: Union[str, UUID_type] = None):

        if df is not None:
            if uuid_curve is not None:
                row = df[df.uuid_curve == uuid_curve]

            elif sample_id is not None:
                rows = df[df.SampleID == sample_id]
                row = rows.iloc[0]

        if row is not None:

            tp = row['ImgPath']
            if isinstance(tp, pd.Series):
                tp = tp.item()

            tiff_path = os.path.join(proj_path, tp)

            pp = row['ImgInfoPath']
            if isinstance(pp, pd.Series):
                pp = pp.item()

            pik_path = os.path.join(proj_path, pp)

            s = row['SampleID']
            if isinstance(s, pd.Series):
                s = s.item()

            roi_state = row['ROI_State']
            if isinstance(roi_state, pd.Series):
                roi_state = roi_state.item()

            if roi_state['roi_type'] == 'CNMFROI':
                cnmf_idx = roi_state['cnmf_idx']
                roi_index = ('cnmf_idx', cnmf_idx)
            else:
                roi_index = None

            self._open_from_dataframe(tiff_path, pik_path, roi_index=roi_index, sample_id=s)
            return

        else:
            raise ValueError('Must specify either df and sample_id/uuid_curve or supply the row (pd.Series)')

    def _open_from_dataframe(self, tiff_path: str, pik_path: str, roi_index: Optional[tuple] = None,
                             sample_id: Optional[str] = None):
        setConfigOptions(imageAxisOrder='row-major')
        self.vi.viewer.workEnv = ViewerWorkEnv.from_pickle(pickle_file_path=pik_path, tiff_path=tiff_path)
        self.vi.update_workEnv()

        self.vi.viewer.workEnv.restore_rois_from_states()
        self.vi.viewer.workEnv.changed_items.clear()

        if roi_index is not None:
            roi_list = self.vi.viewer.workEnv.roi_manager.roi_list
            if roi_index[0] == 'cnmf_idx':
                c_idx = roi_index[1]
                list_index = [r.cnmf_idx for r in roi_list].index(c_idx)
            elif roi_index[0] == 'list_index':
                list_index = roi_index[1]
            else:
                raise ValueError('roi_index must be either a cnmf index or the direct list index of the ROI_List object')

            roi_list.list_widget.setCurrentRow(list_index)
            rm: roi_manager.ModuleGUI = self.get_module('roi_manager')
            if rm.ui.checkBoxShowAll.isChecked():
                rm.ui.checkBoxShowAll.click()
            rm.hide()

            # dirty fix for now to get the plot to draw
            rm.set_curveplot_datatype()

        if sample_id is not None:
            self.vi.viewer.ui.label_curr_img_seq_name.setText(sample_id)

        self.vi.viewer.workEnv.saved = True

    def dragEnterEvent(self, QDragEnterEvent):
        if QDragEnterEvent.mimeData().hasUrls():
            QDragEnterEvent.accept()
        else:
            QDragEnterEvent.ignore()

    def dropEvent(self, ev):
        files = ev.mimeData().urls()
        if len(files) == 1:
            file = files[0].path()
        else:
            return

        ext = os.path.splitext(file)[1]

        self.get_module(file_associations[ext]).file_sink(file)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from PyQt5 import QtCore, QtWidgets
from ..base import BasePlotWidget
from ...variants import PgScatterPlot
from .control_widget import Ui_DockWidgetControls
from ....analysis import Transmission
from ....common.qdialogs import *
import numpy as np
from ....pyqtgraphCore import GraphicsLayoutWidget, mkBrush
from ....pyqtgraphCore.console import ConsoleWidget
from ...utils import get_colormap
from .. import DatapointTracerWidget
import os
from ....common.configuration import console_history_path
from ....common import get_project_manager
from uuid import UUID
import pandas as pd


class ControlDock(QtWidgets.QDockWidget):
    sig_changed = QtCore.pyqtSignal()

    def __init__(self, parent):
        QtWidgets.QDockWidget.__init__(self, parent=parent)
        self.ui = Ui_DockWidgetControls()
        self.ui.setupUi(self)

        self.ui.listWidgetColormap.set_cmap('tab10')

        # signals for live updates
        self.ui.comboBoxDataColumn.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.comboBoxXColumn.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.comboBoxYColumn.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.radioButtonDataColumn.toggled.connect(self.sig_changed.emit)
        self.ui.radioButtonXY.toggled.connect(self.sig_changed.emit)
        self.ui.checkBoxLogX.toggled.connect(self.sig_changed.emit)
        self.ui.checkBoxLogY.toggled.connect(self.sig_changed.emit)
        self.ui.comboBoxColorsColumn.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.listWidgetColormap.currentRowChanged.connect(self.sig_changed.emit)
        self.ui.comboBoxShapesColumn.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.comboBoxUUIDColumn.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.comboBoxDPTCurve.currentTextChanged.connect(self.sig_changed.emit)
        self.ui.spinBoxSpotSize.valueChanged.connect(self.sig_changed.emit)
        self.ui.doubleSpinBoxAlpha.valueChanged.connect(self.sig_changed.emit)

        self.data_controls = [self.ui.comboBoxDataColumn, self.ui.comboBoxXColumn, self.ui.comboBoxYColumn, self.ui.comboBoxDPTCurve]
        self.categorial_controls = [self.ui.comboBoxColorsColumn, self.ui.comboBoxShapesColumn]
        self.uuid_controls = [self.ui.comboBoxUUIDColumn]

    def set_combo_box(self, comboBox: QtWidgets.QComboBox, text: str):
        ix = comboBox.findText(text)
        comboBox.setCurrentIndex(ix)

    def fill_widget(self, data_columns: list, categorical_columns: list, uuid_columns: list):
        # Set the selected options after filling the widget if they are present in the new lists
        p_dcol = self.ui.comboBoxDataColumn.currentText()
        p_xcol = self.ui.comboBoxXColumn.currentText()
        p_ycol = self.ui.comboBoxYColumn.currentText()
        p_colors_col = self.ui.comboBoxColorsColumn.currentText()
        p_shape_col = self.ui.comboBoxShapesColumn.currentText()
        p_ucol = self.ui.comboBoxUUIDColumn.currentText()

        for cb in self.data_controls:
            cb.clear()
            cb.addItems(data_columns)

        if p_dcol in data_columns:
            self.set_combo_box(self.ui.comboBoxDataColumn, p_dcol)
            self.set_combo_box(self.ui.comboBoxXColumn, p_xcol)
            self.set_combo_box(self.ui.comboBoxYColumn, p_ycol)

        for cb in self.categorial_controls:
            cb.clear()
            cb.addItem('------------')
            cb.addItems(categorical_columns)

        if p_colors_col in categorical_columns:
            self.set_combo_box(self.ui.comboBoxColorsColumn, p_colors_col)

        if p_shape_col in self.categorial_controls:
            self.set_combo_box(self.ui.comboBoxShapesColumn, p_shape_col)

        for cb in self.uuid_controls:
            cb.clear()
            cb.addItems(uuid_columns)

        if p_ucol in uuid_columns:
            self.set_combo_box(self.ui.comboBoxUUIDColumn, p_ucol)


class CentralWidget(QtWidgets.QWidget):
    def __init__(self, parent):
        QtWidgets.QWidget.__init__(self, parent=parent)
        self.graphics_view = GraphicsLayoutWidget()
        self.plot_variant = PgScatterPlot(self.graphics_view)  #: Instance of :ref:`PgScatterPlot <API_Variant_PgScatterPlot>` which is the actual plot area
        # self.setCentralWidget(self.graphics_view)

        self.vlayout = QtWidgets.QVBoxLayout()
        self.vlayout.addWidget(self.graphics_view)

        self.status_label = QtWidgets.QLabel(self)
        self.status_label.setMaximumHeight(32)

        self.vlayout.addWidget(self.status_label)

        self.setLayout(self.vlayout)


class ScatterPlotWidget(QtWidgets.QMainWindow, BasePlotWidget):
    drop_opts = []

    def __init__(self):
        QtWidgets.QMainWindow.__init__(self, parent=None)
        BasePlotWidget.__init__(self)
        self.setWindowTitle('Scatter Plot')

        self.central_widget = CentralWidget(self)
        self.plot_variant: PgScatterPlot = self.central_widget.plot_variant  #: the plot variant used for the actual plot area.
        self.graphics_view = self.central_widget.graphics_view
        self.status_label = self.central_widget.status_label
        self.status_label.mousePressEvent = self.show_exception_info

        self.setCentralWidget(self.central_widget)

        self.exception_holder = None  #: Used for holding exceptions that can be viewed by clicking on self.status_label

        self.control_widget = ControlDock(self)
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.control_widget)
        self.control_widget.sig_changed.connect(self.update_plot)
        self.control_widget.ui.pushButtonUpdatePlot.clicked.connect(lambda: self.update_plot())

        self.block_signals_list = [self.control_widget]

        self.plot_opts = None

        self.control_widget.ui.pushButtonSave.clicked.connect(self.save_plot_dialog)
        self.control_widget.ui.pushButtonLoad.clicked.connect(self.open_plot_dialog)

        self.update_live = self.control_widget.ui.checkBoxLiveUpdateFromFlowchart.isChecked()
        self.control_widget.ui.checkBoxLiveUpdateFromFlowchart.toggled.connect(self.set_update_live)

        self.live_datapoint_tracer = DatapointTracerWidget()  #: Instance of :ref:`DatapointTracer <API_DatapointTracer>`
        self.plot_variant.signal_spot_clicked.connect(self.set_current_datapoint)

        self.dockDatapointTracer = QtWidgets.QDockWidget(self)
        self.dockDatapointTracer.setWindowTitle('Datapoint Tracer')
        self.dockDatapointTracer.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable | QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockDatapointTracer.setWidget(self.live_datapoint_tracer)
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.dockDatapointTracer)

        cmd_history_file = os.path.join(console_history_path, 'scatter_plot.pik')

        ns = {'this': self,
              'plot_variant': self.plot_variant,
              'plot_item': self.plot_variant.plot
              }

        txt = ["Namespaces",
               "self as 'this'",
               "plot_variant as 'plot_variant'",
               "plot_variant.plot as 'plot_item'"
               ]

        txt = "\n".join(txt)

        self.console = ConsoleWidget(parent=self, namespace=ns, text=txt, historyFile=cmd_history_file)

        self.dockConsole = QtWidgets.QDockWidget(self)
        self.dockConsole.setWindowTitle('Console')
        self.dockConsole.setFeatures(
            QtWidgets.QDockWidget.DockWidgetFloatable | QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockConsole.setWidget(self.console)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.dockConsole)

    def set_update_live(self, b: bool):
        self.control_widget.ui.checkBoxLiveUpdateFromFlowchart.setChecked(b)
        self.update_live = b

    @BasePlotWidget.signal_blocker
    def set_input(self, transmission: Transmission):
        """Set the input transmission"""
        if (self._transmission is None) or self.update_live:
            super(ScatterPlotWidget, self).set_input(transmission)
            self.update_plot()

    @BasePlotWidget.signal_blocker
    def fill_control_widget(self, **kwargs):
        self.control_widget.fill_widget(**kwargs)

    def get_plot_opts(self, drop: bool = False) -> dict:
        """
         Get the plot options

         :param drop: no drop opts are specified for this plot
         """
        d = {'data_column': self.control_widget.ui.comboBoxDataColumn.currentText(),
             'data_column_radio': self.control_widget.ui.radioButtonDataColumn.isChecked(),
             'x_column': self.control_widget.ui.comboBoxXColumn.currentText(),
             'y_column': self.control_widget.ui.comboBoxYColumn.currentText(),
             'log_x': self.control_widget.ui.checkBoxLogX.isChecked(),
             'log_y': self.control_widget.ui.checkBoxLogY.isChecked(),
             'xy_radio': self.control_widget.ui.radioButtonXY.isChecked(),
             'colors_column': self.control_widget.ui.comboBoxColorsColumn.currentText(),
             'cmap': self.control_widget.ui.listWidgetColormap.current_cmap,
             'shapes_column': self.control_widget.ui.comboBoxShapesColumn.currentText(),
             'uuid_column': self.control_widget.ui.comboBoxUUIDColumn.currentText(),
             'dpt_curve_column': self.control_widget.ui.comboBoxDPTCurve.currentText(),
             'spot_size': self.control_widget.ui.spinBoxSpotSize.value(),
             'spot_alpha': self.control_widget.ui.doubleSpinBoxAlpha.value()
             }

        if drop:
            for k in self.drop_opts:
                d.pop(k)

        return d

    @BasePlotWidget.signal_blocker
    def set_plot_opts(self, opts: dict):
        """Set all plot options from a dict"""
        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxDataColumn, opts['data_column'])
        self.control_widget.ui.radioButtonDataColumn.setChecked(opts['data_column_radio'])

        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxXColumn, opts['x_column'])
        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxYColumn, opts['y_column'])
        self.control_widget.ui.radioButtonXY.setChecked(opts['xy_radio'])

        self.control_widget.ui.checkBoxLogX.setChecked(opts['log_x'])
        self.control_widget.ui.checkBoxLogY.setChecked(opts['log_y'])

        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxColorsColumn, opts['colors_column'])
        self.control_widget.ui.listWidgetColormap.set_cmap(opts['cmap'])

        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxShapesColumn, opts['shapes_column'])
        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxUUIDColumn, opts['uuid_column'])
        self.control_widget.set_combo_box(self.control_widget.ui.comboBoxDPTCurve, opts['dpt_curve_column'])
        self.control_widget.ui.spinBoxSpotSize.setValue(opts['spot_size'])
        self.control_widget.ui.doubleSpinBoxAlpha.setValue(opts['spot_alpha'])

    @exceptions_label('status_label', 'exception_holder', 'Error while setting data', 'Make sure you have selected appropriate columns')
    def update_plot(self):
        """Update the plot data and draw"""

        self.exception_holder = None
        self.status_label.clear()

        self.plot_opts = self.get_plot_opts()

        if self.plot_opts['data_column_radio']:
            data_column = self.plot_opts['data_column']
            data = np.vstack(self.transmission.df[data_column].values)

            if data.shape[1] != 2:
                raise ValueError('Size of arrays in data column must equal 2')
            xs = data[:, 0]
            ys = data[:, 1]

        elif self.plot_opts['xy_radio']:
            xs = self.transmission.df[self.plot_opts['x_column']].values
            ys = self.transmission.df[self.plot_opts['y_column']].values

            if xs.ndim > 1:
                raise ValueError('X Column must contain single values, not arrays')
            if ys.ndim > 1:
                raise ValueError('Y Column must contain single values, not arrays')

        else:
            raise ValueError('Must choose either Data Column or X & Y option')

        self.plot_variant.clear()

        if self.plot_opts['log_x']:
            xs = np.log10(xs)
        if self.plot_opts['log_y']:
            ys = np.log10(ys)

        if self.plot_opts['colors_column'] != '------------':
            colors_map = get_colormap(self.transmission.df[self.plot_opts['colors_column']],
                                       self.plot_opts['cmap'], output='pyqt', alpha=self.plot_opts['spot_alpha'])

            colors = list(map(colors_map.get, self.transmission.df[self.plot_opts['colors_column']]))
            brushes_list = list(map(mkBrush, colors))
        else:
            colors_map = None
            brushes_list = 'r'

        if self.plot_opts['shapes_column'] != '------------':
            shapes = ['o', 's', 't', 'd', '+']
            shapes_labels = self.transmission.df[self.plot_opts['shapes_column']].unique()
            if len(shapes_labels) > len(shapes):
                raise ValueError('Too many labels to set different shapes')
            shapes_map = dict(zip(shapes_labels, shapes))

            shapes_list = list(map(shapes_map.get, self.transmission.df[self.plot_opts['shapes_column']]))
        else:
            shapes_map = None
            shapes_list = 'o'

        self.plot_variant.add_data(xs, ys, self.transmission.df[self.plot_opts['uuid_column']], color=brushes_list,
                                   size=self.plot_opts['spot_size'], symbol=shapes_list)

        if colors_map is not None:
            self.plot_variant.set_legend(colors_map, shapes_map)

    def set_current_datapoint(self, identifier: UUID):
        """Set the UUID of the current datapoint and update the Datapoint Tracer"""
        self.current_datapoint = identifier
        u = str(self.current_datapoint)

        uuid_column = self.plot_opts['uuid_column']

        r = self.transmission.df[self.transmission.df[uuid_column] == u]
        dpt_col = self.plot_opts['dpt_curve_column']

        db_id = r['_BLOCK_']
        if isinstance(db_id, pd.Series):
            db_id = db_id.item()
        ht = self.transmission.history_trace.get_data_block_history(db_id)

        self.live_datapoint_tracer.set_widget(u, dpt_col, row=r, proj_path=get_project_manager().root_dir,
                                              history_trace=ht)

    def show_exception_info(self, mouse_press_ev):
        if self.exception_holder is not None:
            QtWidgets.QMessageBox.warning(self, *self.exception_holder)



from .analysis import *
from .plotting import widgets as plot_widgets
from .plotting import variants as plot_variants
from .common import __version__, utils, configuration
from .common.utils import HdfTools
from .pyqtgraphCore.widgets.MatplotlibWidget import MatplotlibWidget as MPLWidget
__author__ = "Kushal Kolar"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

import numpy as np
from cv2 import EMD, DIST_L2


def emd_1d(u: np.ndarray, v: np.ndarray) -> float:
    """
    Wrapper around OpenCV Earth Mover's Distance (EMD) function to calculate EMD between 1D arrays

    :param u: an array of weights
    :param v: an array of weights
    :return: Earth Mover's Distance between u & v
    """

    a = np.empty((u.size, 2), dtype=np.float32)
    b = np.empty((v.size, 2), dtype=np.float32)

    a[:, 0] = u.astype(np.float32)
    a[:, 1] = np.arange(u.size, dtype=np.float32)

    b[:, 0] = v.astype(np.float32)
    b[:, 1] = np.arange(v.size, dtype=np.float32)

    return EMD(a, b, DIST_L2)[0]



from bokeh.plotting import figure, Figure
from bokeh.models.glyphs import Image, MultiLine
from bokeh.models import HoverTool, ColumnDataSource, TapTool, Slider, TextInput, Select, \
    BoxAnnotation, Patches
from bokeh.models.mappers import LogColorMapper
from bokeh.layouts import gridplot, column, row
import os
import numpy as np
import pandas as pd
from uuid import UUID
from pathlib import Path
from typing import *
import tifffile
from mesmerize.plotting.utils import auto_colormap, map_labels_to_colors
from mesmerize.plotting.web_widgets.core import BokehCallbackSignal, WebPlot
import logging
import pickle


logger = logging.getLogger()


_default_image_figure_params = dict(
    plot_height=500,
    plot_width=500,
    tooltips=[("x", "$x"), ("y", "$y"), ("value", "@image")],
    output_backend='webgl'
)

_default_curve_figure_params = dict(
    plot_height=250,
    plot_width=1000,
    tools='tap,hover,pan,wheel_zoom,box_zoom,reset',
)


def get_numerical_columns(dataframe: pd.DataFrame):
    return [c for c in dataframe.columns if c.startswith('_')]


def get_categorical_columns(dataframe: pd.DataFrame):
    return [c for c in dataframe.columns if not c.startswith('_')]


class DatapointTracer(WebPlot):
    # sig_plot_options_changed = BokehCallbackSignal()
    # sig_frame_changed = BokehCallbackSignal()

    def __init__(
            self,
            parent: WebPlot,
            doc,
            project_path: Union[Path, str],
            tooltip_columns: List[str] = None,
            image_figure_params: dict = None,
            curve_figure_params: dict = None
    ):
        self.sig_plot_options_changed = BokehCallbackSignal()
        self.sig_frame_changed = BokehCallbackSignal()

        WebPlot.__init__(self)

        self.parent = parent

        self.doc = doc
        # self.parent_document: Document = parent_document
        self.project_path: Path = Path(project_path)

        self.frame: np.ndarray = np.empty(0)

        if image_figure_params is None:
            image_figure_params = dict()

        self.image_figure: Figure = figure(
            **{
                **_default_image_figure_params,
                **image_figure_params,
                'output_backend': "webgl"
            }
        )

        # must initialize with some array else it won't work
        empty_img = np.zeros(shape=(100, 100), dtype=np.uint8)

        self.image_glyph: Image = self.image_figure.image(
            image=[empty_img],
            x=0, y=0,
            dw=10, dh=10,
            level="image"
        )

        self.roi_patches_glyph: Patches = self.image_figure.patches(
            xs="xs",
            ys="ys",
            # color="colors",
            color="#ffffff",
            alpha=0.0,
            line_width=2,
            line_alpha=1.0,
            source={
                "xs": [[]],
                "ys": [[]],
                # "colors": ["#ffffff"],
            },
        )

        self.image_figure.grid.grid_line_width = 0

        self.curve_figure: Figure = None

        if curve_figure_params is None:
            curve_figure_params = dict()

        self.curve_figure_params = \
            {
                **_default_curve_figure_params,
                **curve_figure_params
            }

        self.curve_glyph: MultiLine = None

        self.curve_plot_bands: List[BoxAnnotation] = []

        self.tooltip_columns = tooltip_columns
        self.tooltips = None

        if self.tooltip_columns is not None:
            self.tooltips = [(col, f'@{col}') for col in tooltip_columns]

        # self.datatable:

        self.dataframe: pd.DataFrame = None
        self.sample_id: str = None
        self.img_uuid: UUID = None
        self.current_frame: int = -1
        self.tif: tifffile.TiffFile = None
        self.color_mapper: LogColorMapper = None

        self.curve_color_selector = Select(title="Color based on:", value='', options=[''])
        self.curve_color_selector.on_change('value', self.sig_plot_options_changed.trigger)

        self.curve_data_selector = Select(title="Curve data:", value='', options=[''])
        self.curve_data_selector.on_change('value', self.sig_plot_options_changed.trigger)

        self.curve_plot_bands_selector = Select(title="Bands based on:", value='', options=[''])
        self.curve_plot_bands_selector.on_change('value', self.sig_plot_options_changed.trigger)

        ############################################################
        # TEMPORARY
        ############################################################

        # self.button_remove_selection = Button(label="Remove current selection")
        # self.button_remove_selection.on_click(self.remove_sample)
        ############################################################
        ############################################################
        ############################################################

        self.sig_plot_options_changed.connect(self.set_curve)

        self.frame_slider = Slider(start=0, end=1000, value=1, step=10, title="Frame index:")
        self.frame_slider.on_change('value', self.sig_frame_changed.trigger)
        self.sig_frame_changed.connect(self._set_current_frame)

        self.label_filesize: TextInput = TextInput(value='', title='Filesize (GB):')
        self.label_sample_id: TextInput = TextInput(value='', title="SampleID:")

    # def remove_sample(self):
    #     self.parent.dataframe = self.parent.dataframe[
    #         self.parent.dataframe['SampleID'] != self.sample_id
    #     ]
    #
    #     sid = self.parent.dataframe['SampleID'].unique()[0]
    #
    #     self.set_sample(
    #         self.parent.dataframe[self.parent.dataframe['SampleID'] == sid]
    #     )
    #
    #     self.parent.update_glyph()

    def _check_sample(self, dataframe: pd.DataFrame):
        if len(dataframe['SampleID'].unique()) > 1:
            raise ValueError("Greater than one SampleID in the sub-dataframe")

        if len(dataframe['ImgUUID'].unique()) > 1:
            raise ValueError("Greater than one ImgUUID in the sub-dataframe")

        self.dataframe = dataframe.copy(deep=True)
        self.sample_id = dataframe['SampleID'].unique()[0]
        self.img_uuid = dataframe['ImgUUID'].unique()[0]

    @WebPlot.signal_blocker
    def set_sample(self, dataframe: pd.DataFrame):
        """

        :param dataframe: dataframe with values pertaining to one sample
        :return:
        """
        self._check_sample(dataframe)

        fname = f'{self.sample_id}-_-{self.img_uuid}.tiff'
        vid_path = self.project_path.joinpath('images', fname)

        self._set_video(vid_path)
        self._update_plot_options()

        self.set_curve()

        self.label_sample_id.update(value=self.sample_id)

    def _set_video(self, vid_path: Union[Path, str]):
        self.tif = tifffile.TiffFile(vid_path)

        self.current_frame = 0
        self.frame = self.tif.asarray(key=self.current_frame)

        # this is basically used for vmin mvax
        self.color_mapper = LogColorMapper(
            palette=auto_colormap(256, 'gnuplot2', output='bokeh'),
            low=np.nanmin(self.frame),
            high=np.nanmax(self.frame)
        )

        self.image_glyph.data_source.data['image'] = [self.frame]
        self.image_glyph.glyph.color_mapper = self.color_mapper

        # shows the file size in gigabytes
        self.label_filesize.update(value=str(os.path.getsize(vid_path) / 1024 / 1024 / 1024))

    def _get_roi_coors(self, r: pd.Series):
        roi_type = r['roi_type']

        if roi_type == 'ManualROI':
            pos = r['roi_graphics_object_state']['pos']
            points = r['roi_graphics_object_state']['points']

            return points + np.array(pos)

    # @WebPlot.signal_blocker
    def _update_plot_options(self):
        # categorical_columns = get_categorical_columns(self.dataframe)
        logger.debug("Updating plot opts")
        categorical_columns = self.tooltip_columns

        numerical_columns = get_numerical_columns(self.dataframe)

        self.curve_color_selector.update(
            value= \
                self.curve_color_selector.value \
                    if self.curve_color_selector.value in categorical_columns \
                    else categorical_columns[0],
            options=categorical_columns
        )

        self.curve_data_selector.update(
            value= \
                self.curve_data_selector.value \
                    if self.curve_data_selector.value in numerical_columns \
                    else numerical_columns[0],
            options=numerical_columns
        )

        if len(self.parent.transmission.STIM_DEFS) > 0:
            self.curve_plot_bands_selector.update(
                value= \
                    self.curve_plot_bands_selector.value \
                        if self.curve_plot_bands_selector.value in self.parent.transmission.STIM_DEFS \
                        else self.parent.transmission.STIM_DEFS[0],
                options=self.parent.transmission.STIM_DEFS
            )
        else:
            self.curve_plot_bands_selector.update(value='', options=[''])

    def _set_current_frame(self, i: int):
        self.current_frame = i
        frame = self.tif.asarray(key=self.current_frame, maxworkers=20)

        self.image_glyph.data_source.data['image'] = [frame]

    def _get_trimmed_dataframe(self) -> pd.DataFrame:
        """
        Get dataframe for tooltips, JSON serializable.
        """
        return self.dataframe.drop(
            columns=[c for c in self.dataframe.columns if c not in self.tooltip_columns]
        ).copy(deep=True)

    @WebPlot.signal_blocker
    def set_curve(self, *args):
        logger.debug('updating curve')
        logger.debug(self.dataframe)
        data_column = self.curve_data_selector.value
        ys = self.dataframe[data_column].values
        xs = [np.arange(0, v.size) for v in ys]

        self.frame_slider.update(start=0, end=ys[0].size - 1, value=0)

        df = self._get_trimmed_dataframe()

        colors_column = self.curve_color_selector.value
        ncolors = df[colors_column].unique().size

        if ncolors < 11:
            cmap = 'tab10'
        elif 10 < ncolors < 21:
            cmap = 'tab20'
        else:
            cmap = 'hsv'

        src = ColumnDataSource(
            {
                **df,
                'xs': xs,
                'ys': ys,
                'colors': map_labels_to_colors(df[colors_column], cmap, output='bokeh')
            }
        )

        if self.curve_figure is not None:
            self.doc.remove_root(self.curve_figure)
            del self.curve_figure

        # New figure has to be created each time
        self.curve_figure = figure(
            tooltips=self.tooltips,
            **self.curve_figure_params,

        )

        stim_option = self.curve_plot_bands_selector.value
        stim_df = self.dataframe['stim_maps'].iloc[0][0][0].get(stim_option, None)
        if stim_df is not None:
            for ix, stim_period in stim_df.iterrows():
                self.curve_figure.add_layout(
                    BoxAnnotation(
                        left=stim_period['start'],
                        right=stim_period['end'],
                        fill_color=list(stim_period['color'][:-1]),
                        fill_alpha=0.1
                    )
                )

        self.curve_glyph = self.curve_figure.multi_line(
            xs='xs', ys='ys',
            legend=colors_column,
            line_color='colors',
            line_width=2,
            source=src
        )

        # TODO: ROIs
        # # set the ROIs
        # p = pickle.load(
        #     open(
        #         os.path.join(
        #             self.parent.transmission.get_proj_path(),
        #             self.dataframe['ImgInfoPath'].iloc[0]
        #         ),
        #         'rb'
        #     )
        # )
        #
        # roi_coors = self.dataframe['ROI_State'].apply(self._get_roi_coors).values
        #
        # xs = [a[:, 0].tolist() for a in roi_coors]
        # ys = [a[:, 1].tolist() for a in roi_coors]
        #
        # self.roi_patches_glyph.data_source.data['xs'] = xs
        # self.roi_patches_glyph.data_source.data['ys'] = ys
        # colors_list = self.curve_glyph.data_source.data['colors']
        # if len(xs) != len(colors_list):
        #     colors_list = ["#ffffff"] * len(xs)
        # else:
        #     self.roi_patches_glyph.data_source.data['colors'] = colors_list

        self.image_glyph.glyph.dw = self.frame.shape[0]
        self.image_glyph.glyph.dh = self.frame.shape[1]
        self.image_glyph.glyph.x = 0
        self.image_glyph.glyph.y = 0

        # add the new curve plot to the doc root
        self.doc.add_root(self.curve_figure)

        logger.debug(">>>> DATAFRAME IS <<<<<<")
        logger.debug(self.dataframe)

    def set_dashboard(self, figures: List[Figure]):
        logger.info('setting dashboard, this might take a few minutes')
        self.doc.add_root(
            column(
                row(*(f for f in figures), self.image_figure),
                row(
                    self.curve_data_selector,
                    self.curve_color_selector,
                    self.curve_plot_bands_selector
                ),
                row(
                    self.label_sample_id,
                    self.label_filesize,
                ),
                self.frame_slider
            )
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#@author: kushal

#Chatzigeorgiou Group
#Sars International Centre for Marine Molecular Biology

#GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

from PyQt5.QtWidgets import QMessageBox, QFileDialog, QLabel, QWidget
import traceback
from functools import wraps
from . import get_project_manager, is_app
from typing import *
from logging import getLogger
from webbrowser import open_new_tab


"""
Decorators for frequent Qt Dialog GUIs
"""


def present_exceptions(title: str = 'error', msg: str = 'The following error occurred.'):
    """
    Use to catch exceptions and present them to the user in a QMessageBox warning dialog.
    The traceback from the exception is also shown.

    This decorator can be stacked on top of other decorators.

    Example:

    .. code-block: python

            @present_exceptions('Error loading file')
            @use_open_file_dialog('Choose file')
                def select_file(self, path: str, *args):
                    pass


    :param title:       Title of the dialog box
    :param msg:         Message to display above the traceback in the dialog box
    :param help_func:   A helper function which is called if the user clicked the "Help" button
    """
    def catcher(func):
        @wraps(func)
        def fn(self, *args, **kwargs):
            if not is_app():
                return func(self, *args, **kwargs)

            try:
                return func(self, *args, **kwargs)
            except Exception as e:
                tb = traceback.format_exc()

                mb = QMessageBox()
                mb.setIcon(QMessageBox.Warning)
                mb.setWindowTitle(title)
                mb.setText(msg)
                mb.setInformativeText(f"{e.__class__.__name__}: {e}")
                mb.setDetailedText(tb)
                mb.setStandardButtons(
                    QMessageBox.Ok | QMessageBox.Help
                )

                if mb.exec_() == QMessageBox.Help:
                    open_new_tab('http://docs.mesmerizelab.org/en/master/getting_help.html')



                getLogger().info(
                    f"{e.__class__.__name__}: {e}\n"
                    f"{traceback.format_exc()}"
                )
                
        return fn
    return catcher


def exceptions_label(label: str, exception_holder: str = None,
                     title: str = 'error', msg: str = 'The following error occured'):
    """
    Use a label to display an exception instead of a QMessageBox

    :param label: name of a QLabel instance
    :param exception_holder: name of an exception_holder attribute where the exception message is stored.
                             This can be used to view the whole exception when the label is clicked on for example.
    :param title: title supplied for the QMessageBox (if used later)
    :param msg: message supplied for the QMessageBox (if used later)
    """
    def catcher(func):
        @wraps(func)
        def fn(self, *args, **kwargs):
            if not is_app():
                func(self, *args, **kwargs)
                return fn

            try:
                return func(self, *args, **kwargs)
            except Exception as e:
                tb = traceback.format_exc()
                exc = f'{e.__class__.__name__}: {e}'

                qlabel = getattr(self, label)
                qlabel.setText(exc)
                qlabel.setStyleSheet("font-weight: bold; color: red")

                if exception_holder is not None:
                    args = (title, msg + f'\n\n{exc}\n\n{tb}', QMessageBox.Ok)
                    setattr(self, exception_holder, args)
        return fn
    return catcher


def _get_start_dir(start_dir) -> str:
    if start_dir is not None:
        return start_dir
    try:
        d = get_project_manager().root_dir
    except:
        return ''
    else:
        if d is None:
            return ''
        else:
            return d


def use_open_file_dialog(title: str = 'Choose file', start_dir: Union[str, None] = None, exts: List[str] = None):
    """
    Use to pass a file path, for opening, into the decorated function using QFileDialog.getOpenFileName

    :param title:       Title of the dialog box
    :param start_dir:   Directory that is first shown in the dialog box.
    :param exts:        List of file extensions to set the filter in the dialog box
    """
    def wrapper(func):

        @wraps(func)
        def fn(self, *args, **kwargs):
            if 'qdialog' in kwargs.keys():
                if not kwargs['qdialog']:
                    func(self, *args, **kwargs)
                    return fn

            if not is_app():
                func(self, *args, **kwargs)
                return fn

            if exts is None:
                e = []
            else:
                e = exts

            if isinstance(self, QWidget):
                parent = self
            else:
                parent = None

            path = QFileDialog.getOpenFileName(parent, title, _get_start_dir(start_dir), f'({" ".join(e)})')
            if not path[0]:
                return
            path = path[0]
            func(self, path, *args, **kwargs)
        return fn
    return wrapper


def use_save_file_dialog(title: str = 'Save file', start_dir: Union[str, None] = None, ext: str = None):
    """
    Use to pass a file path, for saving, into the decorated function using QFileDialog.getSaveFileName

    :param title:       Title of the dialog box
    :param start_dir:   Directory that is first shown in the dialog box.
    :param exts:        List of file extensions to set the filter in the dialog box
    """
    def wrapper(func):
        @wraps(func)
        def fn(self, *args, **kwargs):
            if not is_app():
                func(self, *args, **kwargs)
                return fn

            if ext is None:
                raise ValueError('Must specify extension')
            if ext.startswith('*'):
                ex = ext[1:]
            else:
                ex = ext

            if isinstance(self, QWidget):
                parent = self
            else:
                parent = None

            path = QFileDialog.getSaveFileName(parent, title, _get_start_dir(start_dir), f'(*{ex})')
            if not path[0]:
                return
            path = path[0]
            if not path.endswith(ex):
                path = f'{path}{ex}'

            func(self, path, *args, **kwargs)

        return fn
    return wrapper


def use_open_dir_dialog(title: str = 'Open directory', start_dir: Union[str, None] = None):
    """
    Use to pass a dir path, to open, into the decorated function using QFileDialog.getExistingDirectory

    :param title:       Title of the dialog box
    :param start_dir:   Directory that is first shown in the dialog box.

    Example:

    .. code-block:: python

        @use_open_dir_dialog('Select Project Directory', '')
        def load_data(self, path, *args, **kwargs):
            my_func_to_do_stuff_and_load_data(path)

    """
    def wrapper(func):
        @wraps(func)
        def fn(self, *args, **kwargs):
            if not is_app():
                func(self, *args, **kwargs)
                return fn

            if isinstance(self, QWidget):
                parent = self
            else:
                parent = None

            path = QFileDialog.getExistingDirectory(parent, title, _get_start_dir(start_dir))
            if not path:
                return
            func(self, path, *args, **kwargs)
        return fn
    return wrapper



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './ui_files/plot_window_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1000, 600)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.graphicsView = PlotWidget(self.centralwidget)
        self.graphicsView.setObjectName("graphicsView")
        self.gridLayout.addWidget(self.graphicsView, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1000, 30))
        self.menubar.setObjectName("menubar")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuAdvanced = QtWidgets.QMenu(self.menubar)
        self.menuAdvanced.setObjectName("menuAdvanced")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidgetTransmissions = QtWidgets.QDockWidget(MainWindow)
        self.dockWidgetTransmissions.setMinimumSize(QtCore.QSize(219, 50))
        self.dockWidgetTransmissions.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidgetTransmissions.setObjectName("dockWidgetTransmissions")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockWidgetTransmissions.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dockWidgetTransmissions)
        self.actionTranmissions_Tree = QtWidgets.QAction(MainWindow)
        self.actionTranmissions_Tree.setCheckable(True)
        self.actionTranmissions_Tree.setChecked(True)
        self.actionTranmissions_Tree.setObjectName("actionTranmissions_Tree")
        self.menuView.addAction(self.actionTranmissions_Tree)
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuAdvanced.menuAction())

        self.retranslateUi(MainWindow)
        self.actionTranmissions_Tree.toggled['bool'].connect(self.dockWidgetTransmissions.setVisible)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.menuView.setTitle(_translate("MainWindow", "&View"))
        self.menuAdvanced.setTitle(_translate("MainWindow", "Adva&nced"))
        self.dockWidgetTransmissions.setWindowTitle(_translate("MainWindow", "T&ranmissions w/ ordered history"))
        self.actionTranmissions_Tree.setText(_translate("MainWindow", "&Tranmissions Tree"))

from ...pyqtgraphCore import PlotWidget



from PyQt5 import QtCore
import tifffile
from uuid import UUID
import traceback
import os
from typing import Optional


class Signals(QtCore.QObject):
    finished = QtCore.pyqtSignal(object)
    error = QtCore.pyqtSignal(str)


class Compressor(QtCore.QRunnable):
    def __init__(self, file_path: str, u: Optional[UUID] = None):
        super(Compressor, self).__init__()
        self.signals = Signals()
        self.u = u
        self.file_path = file_path

    def run(self):
        try:
            tif = tifffile.imread(self.file_path)
            seq = tif.asarray(maxworkers=8)
            filename = os.path.splitext(self.file_path)[0]
            tmp_path = f'{filename}_tmp.tiff'

            tifffile.imsave(tmp_path, data=seq, bigtiff=True, compress=True)

            os.remove(self.file_path)
            os.rename(tmp_path, self.file_path)
        except:
            self.signals.error.emit(traceback.format_exc())
        else:
            self.signals.finished.emit(self.u)
        finally:
            pass



import numpy as np


class BitDepthConverter:
    """
    Convert bit depth of numpy arrays that represent grayscale images.
    """
    @staticmethod
    def create_LUT(levels, source, output):
        """
        Create a LUT for converting from one bit depth to another.
        :param levels:      min and max levels with which to create the LUT
        :type levels:       tuple
        :param source:      bit depth of the source. 16, 32, or 64.
        :type source:       int
        :param output:      desired output bit depth
        :type output:       int
        :return:            8 bit LUT to convert the source's bit depth to 8 bit
        :rtype:             np.ndarray
        """

        accepted_srcs = [16, 32, 64]
        if source not in accepted_srcs:
            raise TypeError('Can only convert from uint16, uint32, or uint64')

        accepted_outs = [8, 16, 32]
        if output not in accepted_outs:
            raise TypeError('Can only output uint8, uint16, or uint32')

        typecode = 'uint' + str(source)
        LUT = np.arange(2**source, dtype=typecode)
        LUT.clip(levels[0], levels[1], out=LUT)
        LUT -= levels[0]
        np.floor_divide(LUT, (levels[1] - levels[0] + 1) / (2**output), out=LUT, casting='unsafe')

        typecode = 'uint' + str(output)
        return LUT.astype(typecode)

    @staticmethod
    def apply_LUT(image, LUT):
        """
        Apply LUT
        :param image:   The image upon which to apply the LUT and change its bit depth
        :type image:    np.ndarray
        :param LUT:     The LUT to use for downscaling the bit depth. Generated by BitDepthConvert.create_LUT
        :type LUT:      np.ndarray
        :return:        Downscaled image with the LUT applied to it
        :rtype:         np.ndarray
        """
        return np.take(LUT, image).astype(np.uint8)
